# -*- encoding: utf-8 -*-
import abc
import functools
from abjad.tools import mathtools
from abjad.tools.pitchtools.Interval import Interval


@functools.total_ordering
class NumberedInterval(Interval):
    '''A numbered interval.

    ::

        >>> numbered_interval = pitchtools.NumberedInterval(-14)
        >>> numbered_interval
        NumberedInterval(-14)

    '''

    ### CLASS VARIABLES ###

    __slots__ = (
        '_number',
        )

    ### INITIALIZER ###

    def __init__(self, arg=None):
        from abjad.tools import pitchtools
        if isinstance(arg, (int, float, int)):
            number = arg
        elif isinstance(arg, pitchtools.Interval):
            number = arg.semitones
        elif arg is None:
            number = 0
        else:
            message = 'can not initialize {}: {!r}.'
            message = message.format(type(self).__name__, arg)
            raise TypeError(message)
        self._number = number

    ### SPECIAL METHODS ###

    def __abs__(self):
        r'''Absolute value of numbered interval.

        Returns new numbered interval.
        '''
        return type(self)(abs(self._number))

    def __add__(self, arg):
        r'''Adds `arg` to numbered interval.

        Returns new numbered interval.
        '''
        if isinstance(arg, type(self)):
            number = self.number + arg.number
            return type(self)(number)
        message = 'must be {}: {!r}.'
        message = message.format(type(self), arg)
        raise TypeError(message)

    def __copy__(self):
        r'''Copies numbered interval.

        Returns new numbered interval.
        '''
        return type(self)(self.number)

    def __eq__(self, arg):
        r'''Is true when `arg` is a numbered interval with number equal to that of
        this numbered interval. Otherwise false.

        Returns boolean.
        '''
        if isinstance(arg, type(self)):
            if self.number == arg.number:
                return True
        return False

    def __float__(self):
        r'''Changes numbered interval to float.

        Returns float.
        '''
        return float(self._number)

    def __hash__(self):
        r'''Hashes numbered interval.

        Returns integer.
        '''
        return hash(repr(self))

    def __int__(self):
        r'''Changes numbered interval to integer.

        Returns integer.
        '''
        return int(self._number)

    def __lt__(self, arg):
        r'''Is true when `arg` is a numbered interval with same direction number
        as this numbered interval and with number greater than that of this
        numbered interval. Otherwise false.

        Returns boolean.
        '''
        if not isinstance(arg, type(self)):
            message = 'must be numbered interval: {!r}.'
            message = message.format(arg)
            raise TypeError(message)
        if not self.direction_number == arg.direction_number:
            message = 'can only compare intervals of same direction.'
            raise ValueError(message)
        return abs(self.number) < abs(arg.number)

    def __neg__(self):
        r'''Negates numbered interval.

        Returns new numbered interval.
        '''
        return type(self)(-self._number)

    def __str__(self):
        r'''String representation of numbered interval.

        Returns string.
        '''
        return self._format_string

    def __sub__(self, arg):
        r'''Subtracts `arg` from numbered interval.

        Returns new numbered interval.
        '''
        if isinstance(arg, type(self)):
            number = self.number - arg.number
            return type(self)(number)
        message = 'must be {}: {!r}.'
        message = message.format(type(self), arg)
        raise TypeError(message)

    ### PRIVATE PROPERTIES ###

    @property
    def _format_string(self):
        return '{}{}'.format(self._direction_symbol, abs(self.number))

    @property
    def _storage_format_specification(self):
        from abjad.tools import systemtools
        positional_argument_values = (
            self.number,
            )
        return systemtools.StorageFormatSpecification(
            self,
            keyword_argument_names=(),
            positional_argument_values=positional_argument_values,
            )

    ### PUBLIC METHODS ###

    @classmethod
    def from_pitch_carriers(cls, pitch_carrier_1, pitch_carrier_2):
        '''Makes numbered interval from `pitch_carrier_1` and
        `pitch_carrier_2`.

        ::

            >>> pitchtools.NumberedInterval.from_pitch_carriers(
            ...     NamedPitch(-2),
            ...     NamedPitch(12),
            ...     )
            NumberedInterval(14)

        Returns numbered interval.
        '''
        from abjad.tools import pitchtools
        # get pitches
        pitch_1 = pitchtools.get_named_pitch_from_pitch_carrier(
            pitch_carrier_1)
        pitch_2 = pitchtools.get_named_pitch_from_pitch_carrier(
            pitch_carrier_2)
        # get difference in semitones
        number = pitchtools.NumberedPitch(pitch_2).pitch_number - \
            pitchtools.NumberedPitch(pitch_1).pitch_number
        # change 1.0, 2.0, ... into 1, 2, ...
        number = mathtools.integer_equivalent_number_to_integer(number)
        # return numbered interval
        return cls(number)

    ### PUBLIC PROPERTIES ###

    @property
    def direction_number(self):
        r'''Direction sign of numbered interval.

        ::

            >>> pitchtools.NumberedInterval(-14).direction_number
            -1

        Returns integer.
        '''
        return mathtools.sign(self.number)

    @property
    def direction_string(self):
        r'''Direction string of named interval.

        ::

            >>> pitchtools.NumberedInterval(-14).direction_string
            'descending'

        Returns ``'ascending'``, ``'descending'`` or none.
        '''
        if self.direction_number == -1:
            return 'descending'
        elif self.direction_number == 0:
            return None
        elif self.direction_number == 1:
            return 'ascending'

    @property
    def number(self):
        r'''Number of numbered interval.

        Returns number.
        '''
        return self._number

    @property
    def numbered_interval_number(self):
        r'''Number of numbered interval.

        ::

            >>> pitchtools.NumberedInterval(-14).numbered_interval_number
            -14

        Returns integer or float.
        '''
        return self._number

    @property
    def semitones(self):
        r'''Semitones corresponding to numbered interval.

        Returns nonnegative number.
        '''
        return self.number


# -*- encoding: utf-8 -*-
import importlib
import os
from abjad.tools import systemtools
from abjad.tools.developerscripttools.DeveloperScript import DeveloperScript


class TestAndRebuildScript(DeveloperScript):
    r'''Tests codebase and rebuilds docs.
    '''

    ### PUBLIC PROPERTIES ###

    @property
    def alias(self):
        r'''Alias of script.

        Returns ``'re'``.
        '''
        return 're'

    @property
    def long_description(self):
        r'''Long description of script.

        Returns string or none.
        '''
        return None

    @property
    def scripting_group(self):
        r'''Scripting group of script.

        Returns none.
        '''
        return None

    @property
    def short_description(self):
        r'''Short description of script.

        Returns string.
        '''
        return 'Run pytest -x, doctest -x and then rebuild the API.'

    @property
    def version(self):
        r'''Version of script.

        Returns float.
        '''
        return 1.0

    ### PUBLIC PROPERTIES ###

    def get_terminal_width(self):
        r'''Borrowed from the py lib.
        '''
        try:
            import termios, fcntl, struct
            call = fcntl.ioctl(0, termios.TIOCGWINSZ,
                               struct.pack('hhhh', 0, 0, 0, 0))
            height, width = struct.unpack('hhhh', call)[:2]
            terminal_width = width
        except (SystemExit, KeyboardInterrupt):
            raise
        except:
            # FALLBACK
            terminal_width = int(os.environ.get('COLUMNS', 80)) - 1
        return terminal_width

    def process_args(self, args):
        r'''Processes `args`.

        Returns none.
        '''
        systemtools.IOManager.clear_terminal()
        if not self.run_doctest(args):
            print()
            if not self.run_pytest(args):
                print()
                self.rebuild_docs(args)

    def rebuild_docs(self, args):
        r'''Rebuilds docs.
        '''
        from abjad.tools import developerscripttools
        developerscripttools.BuildApiScript()(['-X', '-M'])

    def run_doctest(self, args):
        r'''Runs doctest.
        '''
        import doctest

        start_message = ' doctest session starts '
        stop_message = ' {} of {} tests passed, in {} modules '

        print(start_message.center(self.get_terminal_width(), '='))
        print()

        globs = importlib.import_module('abjad').__dict__.copy()
        try:
            globs.update(importlib.import_module('experimental').__dict__)
            globs.update(
                importlib.import_module('experimental.demos').__dict__)
        except ImportError:
            pass
        failed_tests = 0
        passed_tests = 0
        passed_modules = 0
        optionflags = (
            doctest.NORMALIZE_WHITESPACE |
            doctest.ELLIPSIS |
            doctest.REPORT_NDIFF |
            doctest.REPORT_ONLY_FIRST_FAILURE
            )
        for dir_path, dir_names, file_names in os.walk('.'):
            for file_name in sorted(file_names):
                if file_name.endswith('.py') and \
                    not file_name.startswith('test_') and \
                    not file_name == '__init__.py':
                    full_file_name = os.path.abspath(
                        os.path.join(dir_path, file_name))
                    print(os.path.relpath(full_file_name))
                    failure_count, test_count = doctest.testfile(
                        full_file_name,
                        module_relative=False,
                        globs=globs,
                        optionflags=optionflags
                        )
                    if failure_count:
                        failed_tests += failure_count
                        return True
                    passed_modules += 1
                    passed_tests += test_count

        print()
        print(stop_message.format(
            passed_tests - failed_tests,
            passed_tests,
            passed_modules).center(self.get_terminal_width(), '='))

        return False

    def run_pytest(self, args):
        r'''Runs pytest.
        '''
        import pytest
        options = ['-x', '-rf', '.']
        return pytest.main(options)

    def setup_argument_parser(self, parser):
        r'''Sets up argument `parser`.

        Returns none.
        '''
        pass


# -*- encoding: utf-8 -*-
import pytest
from abjad import *


def test_sequencetools_repeat_sequence_to_length_01():
    r'''Repeat list to length.
    '''

    assert sequencetools.repeat_sequence_to_length(list(range(5)), 11) == [0, 1, 2, 3, 4, 0, 1, 2, 3, 4, 0]


def test_sequencetools_repeat_sequence_to_length_02():
    r'''Repeat list to length.
    '''

    sequence_2 = sequencetools.repeat_sequence_to_length([0, -1, -2, -3, -4], 11)
    assert sequence_2 == [0, -1, -2, -3, -4, 0, -1, -2, -3, -4, 0]


def test_sequencetools_repeat_sequence_to_length_03():
    r'''When length is less than length of list
    return only the first length elements of list.
    '''

    when = sequencetools.repeat_sequence_to_length(list(range(5)), 3)
    assert when == [0, 1, 2]


def test_sequencetools_repeat_sequence_to_length_04():
    r'''When length is zero, return an empty list.
    '''

    sequence_2 = sequencetools.repeat_sequence_to_length(list(range(5)), 0)
    assert sequence_2 == []


def test_sequencetools_repeat_sequence_to_length_05():
    r'''List must not be empty.
    '''

    statement = 'sequencetools.repeat_sequence_to_length([], 2)'
    assert pytest.raises(ValueError, statement)


def test_sequencetools_repeat_sequence_to_length_06():
    r'''Optional start index less than length of list.
    '''

    sequence_2 = sequencetools.repeat_sequence_to_length([1, 2, 3], 10, 2)

    assert sequence_2 == [3, 1, 2, 3, 1, 2, 3, 1, 2, 3]


def test_sequencetools_repeat_sequence_to_length_07():
    r'''Optional start index greater than length of list is OK.
    '''

    sequence_2 = sequencetools.repeat_sequence_to_length([1, 2, 3], 10, 100)

    assert sequence_2 == [2, 3, 1, 2, 3, 1, 2, 3, 1, 2]


def test_sequencetools_repeat_sequence_to_length_08():
    r'''Repeat Abjad container to length.
    '''

    container = Container("c'8 d'8 e'8")
    new = sequencetools.repeat_sequence_to_length(container, 5)
    assert format(new) == format(Container("c'8 d'8 e'8 c'8 d'8"))
    assert new[0] is not container[0]



# -*- encoding: utf-8 -*-
from abjad import *


def test_rhythmmakertools_TaleaRhythmMaker__split_sequence_extended_to_weights_01():

    maker = rhythmmakertools.TaleaRhythmMaker
    parts = maker._split_sequence_extended_to_weights(
        [1, 2, 3, 4, 5],
        [7, 7, 7],
        overhang=True,
        )

    assert parts == [[1, 2, 3, 1], [3, 4], [1, 1, 2, 3], [4, 5]]


def test_rhythmmakertools_TaleaRhythmMaker__split_sequence_extended_to_weights_02():

    maker = rhythmmakertools.TaleaRhythmMaker
    parts = maker._split_sequence_extended_to_weights(
        [1, 2, 3, 4, 5],
        [7, 7, 7],
        overhang=False,
        )

    assert parts == [[1, 2, 3, 1], [3, 4], [1, 1, 2, 3]]


# -*- encoding: utf-8 -*-


def all_are_positive_integer_powers_of_two(expr):
    '''Is true when `expr` is a sequence and all elements in `expr`
    are positive integer powers of two.

    ::

        >>> mathtools.all_are_nonnegative_integer_powers_of_two([1, 1, 1, 2, 4, 32, 32])
        True

    Is true when `expr` is an empty sequence:

    ::

        >>> mathtools.all_are_nonnegative_integer_powers_of_two([])
        True

    Otherwise false:

    ::

        >>> mathtools.all_are_nonnegative_integer_powers_of_two(17)
        False

    Returns boolean.
    '''
    from abjad.tools import mathtools

    try:
        return all(
            mathtools.is_positive_integer_power_of_two(x) for x in expr
            )
    except TypeError:
        return False


# -*- encoding: utf-8 -*-
import importlib
import inspect
import sphinx
import types
from docutils import nodes
from sphinx import addnodes


class AbjadDoctestDirective(sphinx.util.compat.Directive):

    has_content = True
    required_arguments = 0
    optional_arguments = 0
    final_argument_whitespace = True
    option_spec = {}

    def run(self):
        self.assert_has_content()
        return []


def doctree_read(app, doctree):

    classes_to_attributes = {}

    def get_unique_parts(parts):
        unique_parts = [parts[0]]
        for part in parts[1:]:
            if part != unique_parts[-1]:
                unique_parts.append(part)
            else:
                break
        return unique_parts

    for desc_node in doctree.traverse(addnodes.desc):
        if desc_node.get('domain') != 'py':
            continue

        signature_node = desc_node.traverse(addnodes.desc_signature)[0]
        module_name = signature_node.get('module')
        object_name = signature_node.get('fullname')
        object_type = desc_node.get('objtype')
        module = importlib.import_module(module_name)

        # Strip 'abjad.tools.' or 'experimental.tools.' from signature.
        # Also strip duplicate class names.
        if object_type in ('function', 'class'):
            addname_node = signature_node.traverse(addnodes.desc_addname)[0]
            text = addname_node[0].astext()
            parts = [x for x in text.split('.') if x]
            parts = get_unique_parts(parts)
            if parts[0] in ('abjad', 'experimental'):
                parts = parts[2:-1]
            text = '{}.'.format('.'.join(parts))
            addname_node[0] = nodes.Text(text)

        if object_type == 'class':
            cls = getattr(module, object_name, None)
            if cls is None:
                continue
            if cls not in classes_to_attributes:
                classes_to_attributes[cls] = {}
                attributes = inspect.classify_class_attrs(cls)
                for attribute in attributes:
                    classes_to_attributes[cls][attribute.name] = attribute
            if inspect.isabstract(cls):
                labelnode = addnodes.only(expr='html')
                labelnode.append(nodes.literal(
                    '',
                    ' abstract ',
                    classes=['attribute', 'abstract'],
                    ))
                signature_node.insert(0, labelnode)

        elif object_type in ('method', 'attribute', 'staticmethod', 'classmethod'):
            object_parts = object_name.split('.')
            cls_name, attr_name = object_name.split('.')
            cls = getattr(module, cls_name, None)
            if cls is None:
                continue
            attr = getattr(cls, attr_name)
            inspected_attr = classes_to_attributes[cls][attr_name]
            label_node = addnodes.only(expr='html')
            defining_class = inspected_attr.defining_class
            if defining_class != cls:
                addname_node = signature_node.traverse(
                    addnodes.desc_addname)[0]
                xref_node = addnodes.pending_xref(
                    '',
                    refdomain='py',
                    refexplicit=True,
                    reftype='class',
                    reftarget='{}.{}'.format(
                        defining_class.__module__,
                        defining_class.__name__,
                        ))
                xref_node.append(nodes.literal(
                    '',
                    '{}'.format(defining_class.__name__),
                    ))
                html_only_class_name_node = addnodes.only(expr='html')
                html_only_class_name_node.append(nodes.Text('('))
                html_only_class_name_node.append(xref_node)
                html_only_class_name_node.append(nodes.Text(').'))
                latex_only_class_name_node = addnodes.only(expr='latex')
                latex_only_class_name_node.append(nodes.Text(
                    '({}).'.format(defining_class.__name__),
                    ))
                addname_node.clear()
                addname_node.append(html_only_class_name_node)
                addname_node.append(latex_only_class_name_node)
                label_node.append(nodes.literal(
                    '',
                    ' inherited ',
                    classes=['attribute', 'inherited'],
                    ))
            if getattr(attr, '__isabstractmethod__', False):
                label_node.append(nodes.literal(
                    '',
                    ' abstract ',
                    classes=['attribute', 'abstract'],
                    ))
            if isinstance(attr, types.FunctionType):
                # remove Sphinx's annotation, so we can use our own.
                signature_node.pop(0)
                label_node.append(nodes.literal(
                    '',
                    ' staticmethod ',
                    classes=['attribute', 'staticmethod'],
                    ))
            elif hasattr(attr, 'im_self') and attr.im_self is not None:
                signature_node.pop(0)
                label_node.append(nodes.literal(
                    '',
                    ' classmethod ',
                    classes=['attribute', 'classmethod'],
                    ))
            signature_node.insert(0, label_node)

def setup(app):
    app.add_directive('doctest', AbjadDoctestDirective)
    app.connect('doctree-read', doctree_read)


# -*- encoding: utf-8 -*-
from abjad import *
from abjad.tools.sievetools import ResidueClass


def test_sievetools_ResidueClass_operator_mixed_01():
    r'''Mixed operators yield nested sieves.
    '''

    rc1 = ResidueClass(4, 0)
    rc2 = ResidueClass(4, 1)
    rc3 = ResidueClass(3, 0)
    rc4 = ResidueClass(3, 1)

    rcsA = rc1 & rc2
    rcsB = rc3 | rc4
    t = rcsA ^ rcsB

    assert isinstance(t, sievetools.Sieve)
    assert t.logical_operator == 'xor'
    assert len(t.rcs) == 2
    assert isinstance(t.rcs[0], sievetools.Sieve)
    assert t.rcs[0].logical_operator == 'and'
    assert isinstance(t.rcs[1], sievetools.Sieve)
    assert t.rcs[1].logical_operator == 'or'
    assert t.rcs[0] is rcsA
    assert t.rcs[1] is rcsB


def test_sievetools_ResidueClass_operator_mixed_02():
    r'''Mixed operators yield nested sieves.
    Sieves with the same operator, merge.
    '''

    rc1 = ResidueClass(4, 0)
    rc2 = ResidueClass(4, 1)
    rc3 = ResidueClass(3, 0)
    rc4 = ResidueClass(3, 1)

    rcsA = rc1 & rc2
    rcsB = rc3 | rc4
    t = rcsA | rcsB

    assert isinstance(t, sievetools.Sieve)
    assert t.logical_operator == 'or'
    assert len(t.rcs) == 3
    assert isinstance(t.rcs[0], sievetools.Sieve)
    assert t.rcs[0].logical_operator == 'and'
    assert isinstance(t.rcs[1], ResidueClass)
    assert isinstance(t.rcs[2], ResidueClass)
    assert t.rcs[0] is rcsA
    assert t.rcs[1] is rc3
    assert t.rcs[2] is rc4


def test_sievetools_ResidueClass_operator_mixed_03():
    r'''Operators combined.
    '''

    t = (ResidueClass(2, 0) ^ ResidueClass(3, 0)) | ResidueClass(3,0)

    assert isinstance(t, sievetools.Sieve)
    assert len(t.rcs) == 2
    assert isinstance(t.rcs[0], sievetools.Sieve)
    assert t.rcs[0].logical_operator == 'xor'
    assert isinstance(t.rcs[1], ResidueClass)
    assert t.get_boolean_train(6) == [1,0,1,1,1,0]
    assert t.get_congruent_bases(6) == [0,2,3,4,6]


def test_sievetools_ResidueClass_operator_mixed_04():
    r'''Operators combined.
    '''

    t = (ResidueClass(2, 0) ^ ResidueClass(3, 0)) | ResidueClass(3,0)

    assert isinstance(t, sievetools.Sieve)
    assert len(t.rcs) == 2
    assert isinstance(t.rcs[0], sievetools.Sieve)
    assert t.rcs[0].logical_operator == 'xor'
    assert isinstance(t.rcs[1], ResidueClass)
    assert t.get_boolean_train(6) == [1,0,1,1,1,0]
    assert t.get_congruent_bases(6) == [0,2,3,4,6]


# -*- encoding: utf-8 -*-
import copy
import inspect
import pytest
import abjad
from abjad.tools import documentationtools
from abjad.tools import lilypondparsertools
from abjad.tools import quantizationtools
from abjad.tools import rhythmtreetools
from abjad.tools import scoretools
from abjad.tools import systemtools


# TODO: make these work
_classes_to_fix = (
    documentationtools.ClassDocumenter,
    documentationtools.Pipe,
    lilypondparsertools.LilyPondParser,
    lilypondparsertools.ReducedLyParser,
    lilypondparsertools.SchemeParser,
    quantizationtools.ParallelJobHandlerWorker,
    rhythmtreetools.RhythmTreeParser,
    systemtools.RedirectedStreams,
    )

classes = documentationtools.list_all_abjad_classes()
@pytest.mark.parametrize('class_', classes)
def test_abjad___deepcopy___01(class_):
    r'''All concrete classes with a storage format can copy.
    '''

    if '_storage_format_specification' not in dir(class_):
        return
    if inspect.isabstract(class_):
        return
    if class_ in _classes_to_fix:
        return
    instance_one = class_()
    instance_two = copy.deepcopy(instance_one)
    instance_one_format = format(instance_one, 'storage')
    instance_two_format = format(instance_two, 'storage')
    if not issubclass(class_, scoretools.Container):
        assert instance_one_format == instance_two_format
    # TODO: eventually this second asset should also pass
    #assert instance_one == instance_two



# -*- encoding: utf-8 -*-
import numbers


def get_named_pitch_from_pitch_carrier(pitch_carrier):
    '''Gets named pitch from `pitch_carrier`.

    ::

        >>> pitch = NamedPitch('df', 5)
        >>> pitch
        NamedPitch("df''")
        >>> pitchtools.get_named_pitch_from_pitch_carrier(pitch)
        NamedPitch("df''")

    ::

        >>> note = Note(('df', 5), (1, 4))
        >>> note
        Note("df''4")
        >>> pitchtools.get_named_pitch_from_pitch_carrier(note)
        NamedPitch("df''")

    ::

        >>> note = Note(('df', 5), (1, 4))
        >>> note.note_head
        NoteHead("df''")
        >>> pitchtools.get_named_pitch_from_pitch_carrier(note.note_head)
        NamedPitch("df''")

    ::

        >>> chord = Chord([('df', 5)], (1, 4))
        >>> chord
        Chord("<df''>4")
        >>> pitchtools.get_named_pitch_from_pitch_carrier(chord)
        NamedPitch("df''")

    ::

        >>> pitchtools.get_named_pitch_from_pitch_carrier(13)
        NamedPitch("cs''")

    Raise value error when `pitch_carrier` carries no pitch.

    Raises value error when `pitch_carrier` carries more than one pitch.

    Returns named pitch.
    '''
    from abjad.tools import scoretools
    from abjad.tools import scoretools
    from abjad.tools import pitchtools

    if isinstance(pitch_carrier, pitchtools.NamedPitch):
        return pitch_carrier
    elif isinstance(pitch_carrier, pitchtools.NumberedPitch):
        return pitchtools.NamedPitch(pitch_carrier)
    elif isinstance(pitch_carrier, numbers.Number):
        return pitchtools.NamedPitch(pitch_carrier)
    elif isinstance(pitch_carrier, scoretools.Note):
        pitch = pitch_carrier.written_pitch
        if pitch is not None:
            return get_named_pitch_from_pitch_carrier(pitch)
        else:
            message = 'no pitch found on {!r}.'
            message = message.format(pitch_carrier)
            raise ValueError(message)
    elif isinstance(pitch_carrier, scoretools.NoteHead):
        pitch = pitch_carrier.written_pitch
        if pitch is not None:
            return get_named_pitch_from_pitch_carrier(pitch)
        else:
            message = 'no pitch found on {!r}.'
            message = message.format(pitch_carrier)
            raise ValueError(message)
    elif isinstance(pitch_carrier, scoretools.Chord):
        pitches = pitch_carrier.written_pitches
        if len(pitches) == 0:
            message = 'no pitch found on {!r}.'
            message = message.format(pitch_carrier)
            raise ValueError(message)
        elif len(pitches) == 1:
            return get_named_pitch_from_pitch_carrier(pitches[0])
        else:
            message = 'multiple pitches found on {!r}.'
            message = message.format(pitch_carrier)
            raise ValueError(message)
    else:
        message = 'pitch carrier {!r} must be pitch, note, note head or chord.'
        message = message.format(pitch_carrier)
        raise TypeError(message)


# -*- encoding: utf-8 -*-
from abjad import *


def test_pitchtools_list_named_pitches_in_expr_01():
    r'''Works with containers.
    '''

    tuplet = scoretools.FixedDurationTuplet(Duration(2, 8), "c'8 d'8 e'8")
    named_pitches = pitchtools.list_named_pitches_in_expr(tuplet)

    assert named_pitches == (
        NamedPitch('c', 4),
        NamedPitch('d', 4),
        NamedPitch('e', 4),
        )


def test_pitchtools_list_named_pitches_in_expr_02():
    r'''Works with spanners.
    '''

    staff = Staff("c'8 d'8 e'8 f'8")
    beam = Beam()
    attach(beam, staff[:])
    named_pitches = pitchtools.list_named_pitches_in_expr(beam)

    assert named_pitches == (
        NamedPitch('c', 4),
        NamedPitch('d', 4),
        NamedPitch('e', 4),
        NamedPitch('f', 4),
        )


def test_pitchtools_list_named_pitches_in_expr_03():
    r'''Works with pitch sets.
    '''

    pitch_set = pitchtools.PitchSet([0, 2, 4, 5])
    named_pitches = pitchtools.list_named_pitches_in_expr(pitch_set)

    assert named_pitches == (
        NamedPitch('c', 4),
        NamedPitch('d', 4),
        NamedPitch('e', 4),
        NamedPitch('f', 4),
        )


def test_pitchtools_list_named_pitches_in_expr_04():
    r'''Works with pitch arrays.
    '''

    array = pitchtools.PitchArray([
        [1, (2, 1), (-1.5, 2)],
        [(7, 2), (6, 1), 1],
        ])

    assert pitchtools.list_named_pitches_in_expr(array) == (
        NamedPitch('d', 4),
        NamedPitch('bqf', 3),
        NamedPitch('g', 4),
        NamedPitch('fs', 4),
        )


def test_pitchtools_list_named_pitches_in_expr_05():
    r'''Works with list or tuple of pitches.
    '''

    named_pitches = (
        NamedPitch(0),
        Note(2, (1, 4)),
        Chord([4, 6, 7], (1, 4)),
        )
    assert pitchtools.list_named_pitches_in_expr(named_pitches) == (
        NamedPitch('c', 4),
        NamedPitch('d', 4),
        NamedPitch('e', 4),
        NamedPitch('fs', 4),
        NamedPitch('g', 4),
        )


# -*- encoding: utf-8 -*-
from abjad.tools.datastructuretools import TreeContainer
from abjad.tools.documentationtools.GraphvizObject import GraphvizObject


class GraphvizNode(TreeContainer, GraphvizObject):
    r'''A Graphviz node.
    '''

    ### INITIALIZER ###

    def __init__(
        self,
        attributes=None,
        children=None,
        name=None,
        ):
        self._children = []
        TreeContainer.__init__(
            self,
            children=children,
            name=name,
            )
        GraphvizObject.__init__(
            self,
            attributes=attributes,
            )
        self._edges = set([])

    ### PRIVATE PROPERTIES ###

    @property
    def _graphviz_format_contributions(self):
        node_def = self._format_value(self.canonical_name)
        attributes = self.attributes
        struct_format_contributions = self._struct_format_contributions
        if struct_format_contributions:
            attributes['label'] = struct_format_contributions
        if len(attributes):
            result = []
            result.extend(self._format_attribute_list(attributes))
            result[0] = '{} {}'.format(node_def, result[0])
            return result
        return [node_def + ';']

    @property
    def _node_class(self):
        from abjad.tools import documentationtools
        prototype = (
            documentationtools.GraphvizField,
            documentationtools.GraphvizGroup,
            )
        return prototype

    @property
    def _struct_format_contributions(self):
        result = []
        for x in self:
            part = x._struct_format_contributions
            if part:
                result.append(part)
        result = ' | '.join(result)
        return result

    ### PUBLIC PROPERTIES ###

    @property
    def all_edges(self):
        r'''Gets edges of this node and those of any field in its field
        subtree.
        '''
        from abjad.tools import documentationtools
        edges = set(self.edges)
        for node in self.nodes[1:]:
            if isinstance(node, documentationtools.GraphvizGroup):
                continue
            edges.update(node.edges)
        return tuple(edges)

    @property
    def canonical_name(self):
        r'''Canonical name of Graphviz node.

        Returns string.
        '''
        if self.name is not None:
            return self.name
        return 'node_' + '_'.join(str(x) for x in self.graph_order)

    @property
    def edges(self):
        r'''Edges of Graphviz node.

        Returns tuple.
        '''
        return tuple(self._edges)


# -*- encoding: utf-8 -*-
import functools
from abjad.tools import mathtools
from abjad.tools.pitchtools.Pitch import Pitch


@functools.total_ordering
class NumberedPitch(Pitch):
    '''A numbered pitch.

    ::

        >>> numbered_pitch = pitchtools.NumberedPitch(13)
        >>> numbered_pitch
        NumberedPitch(13)

    ::

        >>> show(numbered_pitch) # doctest: +SKIP

    '''

    ### CLASS VARIABLES ###

    __slots__ = (
        '_pitch_number',
        )

    ### INITIALIZER ###

    def __init__(self, expr=None):
        from abjad.tools import pitchtools
        if hasattr(expr, 'pitch_number'):
            pitch_number = expr.pitch_number
        elif Pitch.is_pitch_number(expr):
            pitch_number = expr
        elif expr is None:
            pitch_number = 0
        else:
            pitch_number = pitchtools.NamedPitch(expr).pitch_number
        self._pitch_number = mathtools.integer_equivalent_number_to_integer(
            pitch_number)

    ### SPECIAL METHODS ###

    def __add__(self, arg):
        r'''Adds `arg` to numberd pitch.

        Returns new numbered pitch.
        '''
        arg = type(self)(arg)
        semitones = self.pitch_number + arg.pitch_number
        return type(self)(semitones)

    def __eq__(self, arg):
        r'''Is true when `arg` can be coerced to a numbered pitch and when
        this numbered pitch equals `arg`. Otherwise false.

        Returns boolean.
        '''
        try:
            arg = type(self)(arg)
        except (ValueError, TypeError):
            return False
        return self.pitch_number == arg.pitch_number

    def __float__(self):
        r'''Changes numbered pitch to float.

        Returns float.
        '''
        return float(self._pitch_number)

    def __getnewargs__(self):
        r'''Gets new arugments.

        Returns tuple.
        '''
        return (self._pitch_number, )

    def __hash__(self):
        r'''Hashes numbered pitch.

        Returns integer.
        '''
        return hash(repr(self))

    def __int__(self):
        r'''Changes numbered pitch to integer.

        Returns integer.
        '''
        return self._pitch_number

    def __lt__(self, arg):
        r'''Is true when `arg` can be coerced to a numbered pitch and when this
        numbered pitch is less than `arg`. Otherwise false.

        Returns boolean.
        '''
        try:
            arg = type(self)(arg)
        except (ValueError, TypeError):
            return False
        return self.pitch_number < arg.pitch_number

    def __neg__(self):
        r'''Negates numbered pitch.

        Returns new numbered pitch.
        '''
        return type(self)(-self.pitch_number)

    def __str__(self):
        r'''String representation of numbered pitch.

        Returns string.
        '''
        return str(self.pitch_number)

    def __sub__(self, arg):
        r'''Subtracts `arg` from numbered pitch.

        Returns numbered interval.
        '''
        from abjad.tools import pitchtools
        if isinstance(arg, type(self)):
            return pitchtools.NumberedInterval.from_pitch_carriers(
                self, arg)
        else:
            interval = arg
            return pitchtools.transpose_pitch_carrier_by_interval(
                self, -interval)

    ### PRIVATE PROPERTIES ###

    @property
    def _lilypond_format(self):
        return self.pitch_name

    @property
    def _storage_format_specification(self):
        from abjad.tools import systemtools
        positional_argument_values=(
            self.pitch_number,
            )
        return systemtools.StorageFormatSpecification(
            self,
            is_indented=False,
            keyword_argument_names=(),
            positional_argument_values=positional_argument_values,
            )

    ### PUBLIC METHODS ###

    def apply_accidental(self, accidental=None):
        r'''Applies `accidental` to numbered pitch.

        ::

            >>> pitchtools.NumberedPitch(13).apply_accidental('flat')
            NumberedPitch(12)

        Returns new numbered pitch.
        '''
        from abjad.tools.pitchtools.Accidental import Accidental
        accidental = Accidental(accidental)
        semitones = self.pitch_number + accidental.semitones
        return type(self)(semitones)

    def invert(self, axis=None):
        r'''Inverts numberd pitch around `axis`.

        ::

            >>> pitchtools.NumberedPitch(2).invert(0)
            NumberedPitch(-2)

        ::

            >>> pitchtools.NumberedPitch(-2).invert(0)
            NumberedPitch(2)

        ::

            >>> pitchtools.NumberedPitch(2).invert(-3)
            NumberedPitch(-8)

        Returns new numbered pitch.
        '''
        return Pitch.invert(self, axis=axis)

    def multiply(self, n=1):
        r'''Multiplies pitch-class of numbered pitch by `n` and maintains
        octave.

        ::

            >>> pitchtools.NumberedPitch(14).multiply(3)
            NumberedPitch(18)

        Returns new numbered pitch.
        '''
        pitch_class_number = (self.pitch_class_number * n) % 12
        octave_floor = (self.octave_number - 4) * 12
        return type(self)(pitch_class_number + octave_floor)

    def transpose(self, n=0):
        r'''Tranposes numbered pitch by `n` semitones.

        ::

            >>> pitchtools.NumberedPitch(13).transpose(1)
            NumberedPitch(14)

        Returns new numbered pitch.
        '''
        semitones = self.pitch_number + float(n)
        return type(self)(semitones)

    ### PUBLIC PROPERTIES ###

    @property
    def accidental(self):
        r'''Accidental of numbered pitch.

        ::

            >>> pitchtools.NumberedPitchClass(13).accidental
            Accidental('s')

        Returns accidental.
        '''
        from abjad.tools import pitchtools
        return pitchtools.Accidental(self.alteration_in_semitones)

    @property
    def alteration_in_semitones(self):
        r'''Alteration of numbered pitch in semitones.

        ::

            >>> pitchtools.NumberedPitchClass(13).alteration_in_semitones
            1

        Returns integer or float.
        '''
        return self.numbered_pitch_class.alteration_in_semitones

    @property
    def diatonic_pitch_class_name(self):
        r'''Diatonic pitch-class name corresponding to numbered pitch.

        ::

            >>> pitchtools.NumberedPitch(13).diatonic_pitch_class_name
            'c'

        Returns string.
        '''
        return self.numbered_pitch_class.diatonic_pitch_class_name

    @property
    def diatonic_pitch_class_number(self):
        r'''Diatonic pitch-class number of numbered pitch.

        ::

            >>> pitchtools.NumberedPitch(13).diatonic_pitch_class_number
            0

        Returns integer.
        '''
        return self.numbered_pitch_class.diatonic_pitch_class_number

    @property
    def diatonic_pitch_name(self):
        r'''Diatonic pitch name of numbered pitch.

        ::

            >>> pitchtools.NumberedPitch(13).diatonic_pitch_name
            "c''"

        Returns string.
        '''
        return '{}{}'.format(
            self.diatonic_pitch_class_name,
            self.octave.octave_tick_string,
            )

    @property
    def diatonic_pitch_number(self):
        r'''Diatonic pitch-class number corresponding to numbered pitch.

        ::

            >>> pitchtools.NumberedPitch(13).diatonic_pitch_number
            7

        Returns integer.
        '''
        from abjad.tools import pitchtools
        return ((self.octave_number - 4) * 7) + \
            self.diatonic_pitch_class_number

    @property
    def named_pitch(self):
        r'''Named pitch corresponding to numbered pitch.

        ::

            >>> pitchtools.NumberedPitch(13).named_pitch
            NamedPitch("cs''")

        Returns named pitch.
        '''
        from abjad.tools import pitchtools
        return pitchtools.NamedPitch(self)

    @property
    def named_pitch_class(self):
        r'''Named pitch-class corresponding to numbered pitch.

        ::

            >>> pitchtools.NumberedPitch(13).named_pitch_class
            NamedPitchClass('cs')

        Returns named pitch-class.
        '''
        from abjad.tools import pitchtools
        return pitchtools.NamedPitchClass(self)

    @property
    def numbered_pitch(self):
        r'''Numbered pitch.

        ::

            >>> pitchtools.NumberedPitch(13).numbered_pitch
            NumberedPitch(13)

        Returns new numbered pitch.
        '''
        return type(self)(self)

    @property
    def numbered_pitch_class(self):
        r'''Numbered pitch-class corresponding to numbered pitch.

        ::

            >>> pitchtools.NumberedPitch(13).numbered_pitch_class
            NumberedPitchClass(1)

        Returns numbered pitch-class.
        '''
        from abjad.tools import pitchtools
        return pitchtools.NumberedPitchClass(self)

    @property
    def octave(self):
        r'''Octave of numbered pitch.

        ::

            >>> pitchtools.NumberedPitch(13).octave
            Octave(5)

        Returns octave.
        '''
        from abjad.tools import pitchtools
        return pitchtools.Octave(self.octave_number)

    @property
    def octave_number(self):
        r'''Octave number of numbered pitch.

        ::

            >>> pitchtools.NumberedPitch(13).octave_number
            5

        Returns integer.
        '''
        return self._pitch_number // 12 + 4

    @property
    def pitch_class_name(self):
        r'''Pitch-class name of numbered pitch.

        ::

            >>> pitchtools.NumberedPitch(13).pitch_class_name
            'cs'

        Returns string.
        '''
        return self.numbered_pitch_class.pitch_class_name

    @property
    def pitch_class_number(self):
        r'''Pitch-class number of numbered pitch.

        ::

            >>> pitchtools.NumberedPitch(13).pitch_class_number
            1

        Returns integer or float.
        '''
        return self._pitch_number % 12

    @property
    def pitch_class_octave_label(self):
        r'''Pitch-class / octave label of numbered pitch.

        ::

            >>> pitchtools.NumberedPitch(13).pitch_class_octave_label
            'C#5'

        Returns string.
        '''
        return '{}{}{}'.format(
            self.diatonic_pitch_class_name.upper(),
            self.accidental.symbolic_string,
            self.octave_number,
            )

    @property
    def pitch_name(self):
        r'''Pitch name corresponding to numbered pitch.

        ::

            >>> pitchtools.NumberedPitch(13).pitch_name
            "cs''"

        Returns string.
        '''
        return '{}{}'.format(
            self.numbered_pitch_class.pitch_class_name,
            self.octave.octave_tick_string,
            )

    @property
    def pitch_number(self):
        r'''Pitch number of numbered pitch.

        ::

            >>> pitchtools.NumberedPitch(13).pitch_number
            13

        Returns number.
        '''
        return self._pitch_number


# -*- encoding: utf-8 -*-
import pytest
from abjad.tools import stringtools


def test_arg_to_tridirectional_ordinal_constant_01():

    assert stringtools.arg_to_tridirectional_ordinal_constant('^') is Up
    assert stringtools.arg_to_tridirectional_ordinal_constant('-') is Center
    assert stringtools.arg_to_tridirectional_ordinal_constant('_') is Down
    assert stringtools.arg_to_tridirectional_ordinal_constant(Up) is Up
    assert stringtools.arg_to_tridirectional_ordinal_constant('default') is Center
    assert stringtools.arg_to_tridirectional_ordinal_constant('neutral') is Center
    assert stringtools.arg_to_tridirectional_ordinal_constant(Down) is Down
    assert stringtools.arg_to_tridirectional_ordinal_constant(1) is Up
    assert stringtools.arg_to_tridirectional_ordinal_constant(0) is Center
    assert stringtools.arg_to_tridirectional_ordinal_constant(-1) is Down
    assert stringtools.arg_to_tridirectional_ordinal_constant(None) is None
    assert pytest.raises(ValueError,
        "stringtools.arg_to_tridirectional_ordinal_constant('foo')")


# -*- encoding: utf-8 -*-
from abjad.tools.datastructuretools.TypedList import TypedList
from abjad.tools.markuptools.Markup import Markup


class MarkupInventory(TypedList):
    '''Abjad model of an ordered list of markup:

    ::

        >>> inventory = markuptools.MarkupInventory(['foo', 'bar'])

    ::

        >>> inventory
        MarkupInventory([Markup(contents=('foo',)), Markup(contents=('bar',))])

    Markup inventories implement the list interface and are mutable.
    '''

    ### CLASS VARIABLES ###

    __slots__ = (
        )

    ### SPECIAL METHODS ###

    # TODO: add verifiable code in doctest
    def __illustrate__(self):
        r'''Illustrates markup inventory.

        ..  container:: example

            ::

                >>> inventory = markuptools.MarkupInventory(['foo', 'bar'])
                >>> show(inventory) # doctest: +SKIP

        Returns LilyPond file.
        '''
        from abjad.tools import lilypondfiletools
        from abjad.tools import markuptools
        lilypond_file = lilypondfiletools.make_basic_lilypond_file()
        for name in ('layout', 'paper', 'score'):
            block = lilypond_file[name]
            lilypond_file.items.remove(block)
        lilypond_file.header_block.tagline = markuptools.Markup('""')
        for item in self:
            lilypond_file.items.append(item)
        return lilypond_file

    ### PRIVATE PROPERTIES ###

    @property
    def _attribute_manifest(self):
        from abjad.tools import systemtools
        return systemtools.AttributeManifest()

    @property
    def _item_callable(self):
        return Markup


# -*- encoding: utf-8 -*-
from abjad.tools.abctools.AbjadObject import AbjadObject


class AttributeDetail(AbjadObject):
    r'''Attribute detail.
    '''

    ### CLASS VARIABLES ###

    __slots__ = (
        '_display_string',
        '_editor',
        '_is_keyword',
        '_command',
        '_name',
        )

    ### INITIALIZER ###

    def __init__(
        self,
        display_string=None,
        editor=None,
        is_keyword=True,
        command=None,
        name=None,
        ):
        if not display_string and name:
            display_string = name.replace('_', ' ')
        self._display_string = display_string
        self._editor = editor
        self._is_keyword = is_keyword
        self._command = command
        self._name = name

    ### SPECIAL METHODS ###

    def __repr__(self):
        r'''Gets interpreter representation of attribute detail.

        Returns string.
        '''
        parts = []
        if self.display_string:
            parts.append(self.display_string)
        if self.command:
            parts.append(self.command)
        if self.editor:
            parts.append(self.editor.__name__)
        parts = ', '.join(parts)
        return '{}({})'.format(type(self).__name__, parts)

    ### PUBLIC PROPERTIES ###

    @property
    def command(self):
        r'''Gets menu key of attribute detail.

        Returns string.
        '''
        return self._command

    @property
    def display_string(self):
        r'''Gets display string of attribute detail.

        Returns string.
        '''
        return self._display_string

    @property
    def editor(self):
        r'''Gets editor callable of attribute detail.

        Returns callable.
        '''
        return self._editor

    @property
    def is_keyword(self):
        r'''Is true when attribute detail is keyword.

        Returns boolean.
        '''
        return self._is_keyword

    @property
    def name(self):
        r'''Gets name of attribute detail.

        Returns string.
        '''
        return self._name


# -*- encoding: utf-8 -*-
import pytest
from abjad import *


def test_durationtools_Duration_to_clock_string_01():

    assert Duration(0).to_clock_string() == "0'00\""
    assert Duration(4).to_clock_string() == "0'04\""
    assert Duration(20).to_clock_string() == "0'20\""
    assert Duration(60).to_clock_string() == "1'00\""
    assert Duration(120).to_clock_string() == "2'00\""
    assert Duration(240).to_clock_string() == "4'00\""


def test_durationtools_Duration_to_clock_string_02():

    assert Duration(Fraction(0)).to_clock_string() == "0'00\""
    assert Duration(Fraction(3, 2)).to_clock_string() == "0'01\""
    assert Duration(Fraction(89, 14)).to_clock_string() == "0'06\""
    assert Duration(Fraction(116, 19)).to_clock_string() == "0'06\""
    assert Duration(Fraction(140, 3)).to_clock_string() == "0'46\""
    assert Duration(Fraction(180, 3)).to_clock_string() == "1'00\""


def test_durationtools_Duration_to_clock_string_03():

    assert pytest.raises(ValueError, 'Duration(-1.5).to_clock_string()')


def test_durationtools_Duration_to_clock_string_04():

    assert Duration(0).to_clock_string(escape_ticks=True) == "0'00\\\""
    assert Duration(4).to_clock_string(escape_ticks=True) == "0'04\\\""
    assert Duration(20).to_clock_string(escape_ticks=True) == "0'20\\\""
    assert Duration(60).to_clock_string(escape_ticks=True) == "1'00\\\""
    assert Duration(120).to_clock_string(escape_ticks=True) == "2'00\\\""
    assert Duration(240).to_clock_string(escape_ticks=True) == "4'00\\\""


# -*- encoding: utf-8 -*-
from abjad import *


def test_mathtools_all_are_positive_integer_equivalent_numbers_01():

    assert mathtools.all_are_positive_integer_equivalent_numbers(
        [Fraction(4, 2), 2.0, 2])


def test_mathtools_all_are_positive_integer_equivalent_numbers_02():

    assert not mathtools.all_are_positive_integer_equivalent_numbers(
        [Fraction(5, 2), 2.5, -2, 0])


# -*- encoding: utf-8 -*-
import os
from abjad import *
configuration = systemtools.AbjadConfiguration()
ly_path = os.path.join(
    configuration.abjad_directory,
    'test.ly',
    )
pdf_path = os.path.join(
    configuration.abjad_directory,
    'test.pdf',
    )
candidate_pdf_path = os.path.join(
    configuration.abjad_directory,
    'test.candidate.pdf',
    )
paths = [ly_path, pdf_path, candidate_pdf_path]


def test_agenttools_PersistenceAgent_as_pdf_01():
    r'''When candidacy is true, agent persists PDF file when no
    PDF file exists.
    '''

    note = Note("c'4")
    with systemtools.FilesystemState(remove=paths):
        result = persist(note).as_pdf(pdf_path, candidacy=True)
        assert os.path.isfile(pdf_path)
        assert isinstance(result, tuple)


def test_agenttools_PersistenceAgent_as_pdf_02():
    r'''When candidacy is true, agent refuses to persist PDF file when
    equivalent PDF file already exists.
    '''

    note = Note("c'4")
    with systemtools.FilesystemState(remove=paths):
        result = persist(note).as_pdf(pdf_path, candidacy=True)
        assert os.path.isfile(pdf_path)
        assert isinstance(result, tuple)
        os.remove(ly_path)
        result = persist(note).as_pdf(pdf_path, candidacy=True)
        assert result == False


def test_agenttools_PersistenceAgent_as_pdf_03():
    r'''When candidacy is true, agent persists PDF file when
    nonequivalent PDF file already exists.
    '''

    note = Note("c'4")
    with systemtools.FilesystemState(remove=paths):
        with open(pdf_path, 'w') as file_pointer:
            file_pointer.write('extra text')
        with open(pdf_path, 'r') as file_pointer:
            lines = file_pointer.readlines()
        contents = ''.join(lines)
        assert 'extra text' in contents
        result = persist(note).as_pdf(pdf_path, candidacy=True)
        assert isinstance(result, tuple)
        with open(pdf_path, 'rb') as file_pointer:
            lines = file_pointer.readlines()
        contents = b''.join(lines)
        assert b'extra text' not in contents



# -*- encoding: utf-8 -*-
import abc
import argparse
import os
from abjad.tools import abctools
from abjad.tools import stringtools


class DeveloperScript(abctools.AbjadObject):
    r'''Object-oriented model of a developer script.

    `DeveloperScript` is the abstract parent from which concrete developer
    scripts inherit.

    Developer scripts can be called from the command line, generally via the
    `ajv` command.

    Developer scripts can be instantiated by other developer scripts in order
    to share functionality.
    '''

    ### CLASS VARIABLES ###

    __slots__ = (
        '_argument_parser',
        )

    ### INITIALIZER ###

    def __init__(self):
        parser = self._argument_parser = argparse.ArgumentParser(
            description=self.short_description,
            epilog=self.long_description,
            formatter_class=argparse.RawDescriptionHelpFormatter,
            prog=self.program_name,
            )
        parser.add_argument('--version', action='version',
            version='%(prog)s {}'.format(self.version))
        self._argument_parser = parser
        self.setup_argument_parser(parser)

    ### SPECIAL METHODS ###

    def __call__(self, args=None):
        r'''Calls developer script.

        Returns none.
        '''
        if args is None:
            args = self.argument_parser.parse_args()
        else:
            if isinstance(args, str):
                args = args.split()
            elif not isinstance(args, (list, tuple)):
                raise ValueError
            args = self.argument_parser.parse_args(args)
        self.process_args(args)

    def __getstate__(self):
        r'''Gets object state.
        '''
        return {}

    ### PUBLIC PROPERTIES ###

    @property
    def alias(self):
        r'''The alias to use for the script, useful only if the script defines
        an abj-dev scripting group as well.
        '''
        return None

    @property
    def argument_parser(self):
        r'''The script's instance of argparse.ArgumentParser.
        '''
        return self._argument_parser

    @property
    def colors(self):
        r'''Colors.

        Returns dictionary.
        '''
        return {
            'BLUE': '\033[94m',
            'RED': '\033[91m',
            'GREEN': '\033[92m',
            'END': '\033[0m',
            }

    @property
    def formatted_help(self):
        r'''Formatted help of developer script.
        '''
        return self._argument_parser.format_help()

    @property
    def formatted_usage(self):
        r'''Formatted usage of developer script.
        '''
        return self._argument_parser.format_usage()

    @property
    def formatted_version(self):
        r'''Formatted version of developer script.
        '''
        return self._argument_parser.format_version()

    @abc.abstractproperty
    def long_description(self):
        r'''The long description, printed after arguments explanations.
        '''
        raise NotImplementedError

    @property
    def program_name(self):
        r'''The name of the script, callable from the command line.
        '''
        name = type(self).__name__[:type(self).__name__.rfind('Script')]
        return stringtools.upper_camel_case_to_space_delimited_lowercase(
            name).replace(' ', '-')

    @property
    def scripting_group(self):
        r'''Scripting subcommand group of script.
        '''
        return None

    @abc.abstractproperty
    def short_description(self):
        r'''Short description of the script, printed before arguments
        explanations.

        Also used as a summary in other contexts.
        '''
        raise NotImplementedError

    @abc.abstractproperty
    def version(self):
        r'''Version number of developer script.
        '''
        raise NotImplementedError

    ### PUBLIC METHODS ###

    @abc.abstractmethod
    def process_args(self, args):
        r'''Processes `args`.
        '''
        pass

    @abc.abstractmethod
    def setup_argument_parser(self):
        r'''Sets up argument parser.
        '''
        pass


# -*- encoding: utf-8 -*-
import abc
from abjad.tools.abctools.AbjadObject import AbjadObject


class TypedCollection(AbjadObject):
    r'''Abstract base class for typed collections.
    '''

    ### CLASS VARIABLES ###

    __slots__ = (
        '_collection',
        '_item_class',
        )

    ### INITIALIZER ###

    @abc.abstractmethod
    def __init__(self, items=None, item_class=None):
        assert isinstance(item_class, (type(None), type))
        self._item_class = item_class

    ### SPECIAL METHODS ###

    def __contains__(self, item):
        r'''Is true when typed collection container `item`.
        Otherwise false.

        Returns boolean.
        '''
        try:
            item = self._item_callable(item)
        except ValueError:
            return False
        return self._collection.__contains__(item)

    def __eq__(self, expr):
        r'''Is true when `expr` is a typed collection with items that compare
        equal to those of this typed collection. Otherwise false.

        Returns boolean.
        '''
        if isinstance(expr, type(self)):
            return self._collection == expr._collection
        elif isinstance(expr, type(self._collection)):
            return self._collection == expr
        return False

    def __format__(self, format_specification=''):
        r'''Formats typed collection.

        Set `format_specification` to `''` or `'storage'`.
        Interprets `''` equal to `'storage'`.

        Returns string.
        '''
        from abjad.tools import systemtools
        if format_specification in ('', 'storage'):
            return systemtools.StorageFormatManager.get_storage_format(self)
        return str(self)

    def __getnewargs__(self):
        r'''Gets new arguments.

        Returns tuple.
        '''
        return (self._collection, self.item_class)

    def __hash__(self):
        r'''Hashes typed collection.

        Required to be explicitely re-defined on Python 3 if __eq__ changes.

        Returns integer.
        '''
        return super(TypedCollection, self).__hash__()

    def __iter__(self):
        r'''Iterates typed collection.

        Returns generator.
        '''
        return self._collection.__iter__()

    def __len__(self):
        r'''Length of typed collection.

        Returns nonnegative integer.
        '''
        return len(self._collection)

    def __ne__(self, expr):
        r'''Is true when `expr` is not a typed collection with items equal to
        this typed collection. Otherwise false.

        Returns boolean.
        '''
        return not self.__eq__(expr)

    ### PRIVATE METHODS ###

    def _on_insertion(self, item):
        r'''Override to operate on item after insertion into collection.
        '''
        pass

    def _on_removal(self, item):
        r'''Override to operate on item after removal from collection.
        '''
        pass

    ### PRIVATE PROPERTIES ###

    @property
    def _item_callable(self):
        def coerce_(x):
            if isinstance(x, self._item_class):
                return x
            return self._item_class(x)
        if self._item_class is None:
            return lambda x: x
        return coerce_

    @property
    def _repr_specification(self):
        from abjad.tools import systemtools
        manager = systemtools.StorageFormatManager
        names =  manager.get_signature_keyword_argument_names(self)
        keyword_argument_names = list(names)
        if 'items' in keyword_argument_names:
            keyword_argument_names.remove('items')
        keyword_argument_names = tuple(keyword_argument_names)
        positional_argument_values=(
            self._collection,
            )
        return systemtools.StorageFormatSpecification(
            self,
            is_indented=False,
            keyword_argument_names=keyword_argument_names,
            positional_argument_values=positional_argument_values,
            )

    @property
    def _storage_format_specification(self):
        from abjad.tools import systemtools
        manager = systemtools.StorageFormatManager
        names = manager.get_signature_keyword_argument_names(self)
        keyword_argument_names = list(names)
        if 'items' in keyword_argument_names:
            keyword_argument_names.remove('items')
        keyword_argument_names = tuple(keyword_argument_names)
        positional_argument_values=(
            self._collection,
            )
        return systemtools.StorageFormatSpecification(
            self,
            keyword_argument_names=keyword_argument_names,
            positional_argument_values=positional_argument_values,
            )

    ### PUBLIC PROPERTIES ###

    @property
    def item_class(self):
        r'''Item class to coerce items into.
        '''
        return self._item_class

    @property
    def items(self):
        r'''Gets collection items.
        '''
        return [x for x in self]


# -*- encoding: utf-8 -*-


def format_input_lines_as_regression_test(input_lines, tab_width=3):
    r"""Formats `input_lines` as regression test.

    ..  container:: example

        ::

            >>> input_lines = '''
            ... staff = Staff("c'8 d'8 e'8 f'8")
            ... beam = spannertools.Beam()
            ... attach(beam, staff.select_leaves())
            ... f(staff)
            ...
            ... scoretools.FixedDurationTuplet(Duration(2, 8), staff[:3])
            ... f(staff)
            ... '''

        ::

            >>> stringtools.format_input_lines_as_regression_test(input_lines) # doctest: +SKIP

                staff = Staff("c'8 d'8 e'8 f'8")
                beam = spannertools.Beam()
                attach(beam, staff.select_leaves())

                r'''
                \new Staff {
                    c'8 [
                    d'8
                    e'8
                    f'8 ]
                }
                '''

                scoretools.FixedDurationTuplet(Duration(2, 8), staff[:3])

                r'''
                \new Staff {
                    \times 2/3 {
                        c'8 [
                        d'8
                        e'8
                    }
                    f'8 ]
                }

                assert select(staff).is_well_formed()
                assert format(staff) == "\\new Staff {
                    \n\t\\times 2/3 {\n\t\tc'8 [\n\t\td'8\n\t\te'8\n\t}\n\tf'8 ]\n}"
                '''

    Formats expressions intelligently.

    Treats blank lines intelligently.

    Removes line-final hash characters.

    Use when writing tests.

    Returns string.
    """

    tab = ' ' * tab_width
    start = tab
    lines = input_lines.split('\n')
    most = ''
    for i, line in enumerate(lines):
        if line == '':
            print('')
        elif line.startswith('f('):
            last_format_line = line
            print('')
            print(tab + "r'''")
            print(_replace_line_with_tabbed_format(tab, most, line))
            print(tab + "'''")
        elif line.endswith('#'):
            line = line.replace('#', '')
            most += line + '\n'
            print(start + line)
        else:
            most += line + '\n'
            print(start + line)
    last_variable = last_format_line[2:-1]
    print(tab + 'assert select(%s).is_well_formed()' % last_variable)
    format_string = _replace_line_with_format(most, last_format_line)
    format_string = repr(format_string)
    print(tab + 'assert format(%s) == %s' % (last_variable, format_string))


def _replace_line_with_format(most_lines, last_line):
    header = 'from abjad import *\n'
    most_lines = header + most_lines
    exec(most_lines)
    last_variable = last_line[2:-1]
    exec(most_lines)
    exec('__x = %s.format' % last_variable)
    return __x


def _replace_line_with_tabbed_format(tab, most_lines, last_line):
    __x = _replace_line_with_format(most_lines, last_line)
    format_lines = __x.split('\n')
    format_lines = [tab + format_line for format_line in format_lines]
    format_str = '\n'.join(format_lines)
    return format_str


# -*- encoding: utf-8 -*-
import collections
import types


def list_all_abjad_functions(modules=None):
    r'''Lists all public functions defined in Abjad.

    ::

        >>> all_functions = documentationtools.list_all_abjad_functions()

    '''
    from abjad import abjad_configuration
    from abjad.tools import documentationtools
    all_functions = set()
    paths = []
    if modules is None:
        paths.append(abjad_configuration.abjad_directory)
    elif isinstance(modules, types.ModuleType):
        paths.extend(modules.__path__)
    elif isinstance(modules, collections.Iterable):
        for module in modules:
            if isinstance(module, types.ModuleType):
                paths.extend(module.__path__)
    for path in paths:
        function_documenter = documentationtools.FunctionCrawler(
            path,
            root_package_name='abjad',
            )
        for x in function_documenter():
            all_functions.add(x)
    return list(all_functions)


# -*- encoding: utf-8 -*-


def is_fraction_equivalent_pair(expr):
    r'''Is true when `expr` is an integer-equivalent pair of numbers
    excluding ``0`` as the second term.

    ::

        >>> mathtools.is_fraction_equivalent_pair((2, 3))
        True

    Otherwise false:

    ::

        >>> mathtools.is_fraction_equivalent_pair((2, 0))
        False

    Returns boolean.
    '''
    from abjad.tools import mathtools

    return mathtools.is_integer_equivalent_pair(expr) and not expr[1] == 0


# -*- encoding: utf-8 -*-
import os
import re
import shutil
import tempfile
from abjad.tools import abctools


class PersistenceAgent(abctools.AbjadObject):
    r'''A wrapper around the Abjad persistence methods.

    ..  container:: example

        ::

            >>> staff = Staff("c'4 e'4 d'4 f'4")
            >>> show(staff) # doctest: +SKIP

        ::

            >>> persist(staff)
            PersistenceAgent(client=Staff("c'4 e'4 d'4 f'4"))

    '''

    ### CLASS VARIABLES ###

    __slots__ = (
        '_client',
        )

    _png_page_pattern = re.compile(r'.+page(\d+)\.png')

    ### INITIALIZER ###

    def __init__(self, client=None):
        self._client = client

    ### PRIVATE PROPERTIES ###

    @property
    def client(self):
        r'''Client of persistence agent.

        Returns selection or component.
        '''
        return self._client

    ### PUBLIC METHODS ###

    def as_ly(
        self, 
        ly_file_path=None, 
        candidacy=False,
        illustrate_function=None, 
        **kwargs
        ):
        r'''Persists client as LilyPond file.

        Autogenerates file path when `ly_file_path` is none.

        ::

            >>> staff = Staff("c'4 e'4 d'4 f'4")
            >>> for x in persist(staff).as_ly('~/example.ly'): # doctest: +SKIP
            ...     x
            ...
            '/Users/josiah/Desktop/test.ly'
            0.04491996765136719

        When `candidacy` is true, writes LilyPond output only when LilyPond 
        output would differ from any existing output file.

        When `candidacy` is false, writes LilyPond output even if LilyPond
        output would not differ from any existing output file.

        Returns output path and elapsed formatting time when LilyPond output is
        written.

        Returns false when when LilyPond output is not written due to 
        `candidacy` being set.
        '''
        from abjad import abjad_configuration
        from abjad.tools import systemtools
        if illustrate_function is None:
            assert '__illustrate__' in dir(self._client)
            illustrate_function = self._client.__illustrate__
        illustration = illustrate_function(**kwargs)
        if ly_file_path is None:
            ly_file_name = systemtools.IOManager.get_next_output_file_name()
            ly_file_path = os.path.join(
                abjad_configuration.abjad_output_directory,
                ly_file_name,
                )
        else:
            ly_file_path = os.path.expanduser(ly_file_path)
        assert ly_file_path.endswith('.ly'), ly_file_path
        timer = systemtools.Timer()
        with timer:
            lilypond_format = format(illustration, 'lilypond')
        abjad_formatting_time = timer.elapsed_time
        directory = os.path.dirname(ly_file_path)
        systemtools.IOManager._ensure_directory_existence(directory)
        if not os.path.exists(ly_file_path) or not candidacy:
            with open(ly_file_path, 'w') as file_pointer:
                file_pointer.write(lilypond_format)
            return ly_file_path, abjad_formatting_time
        base, extension = os.path.splitext(ly_file_path)
        candidate_path = base + '.candidate' + extension
        with systemtools.FilesystemState(remove=[candidate_path]):
            with open(candidate_path, 'w') as file_pointer:
                file_pointer.write(lilypond_format)
            if systemtools.TestManager.compare_files(
                ly_file_path, 
                candidate_path,
                ):
                return False
            else:
                shutil.move(candidate_path, ly_file_path)
                return ly_file_path, abjad_formatting_time
        
    def as_midi(self, midi_file_path=None, remove_ly=False, **kwargs):
        r'''Persists client as MIDI file.

        Autogenerates file path when `midi_file_path` is none.

        ::

            >>> staff = Staff("c'4 e'4 d'4 f'4")
            >>> for x in persist(staff).as_midi(): # doctest: +SKIP
            ...     x
            ...
            '/Users/josiah/.abjad/output/1415.midi'
            0.07831692695617676
            1.0882699489593506

        Returns output path, elapsed formatting time and elapsed rendering
        time.
        '''
        from abjad.tools import lilypondfiletools
        from abjad.tools import systemtools
        assert '__illustrate__' in dir(self._client)
        illustration = self._client.__illustrate__(**kwargs)
        assert hasattr(illustration, 'score_block')
        block = lilypondfiletools.Block(name='midi')
        illustration.score_block.items.append(block)
        if midi_file_path is not None:
            midi_file_path = os.path.expanduser(midi_file_path)
            without_extension = os.path.splitext(midi_file_path)[0]
            ly_file_path = '{}.ly'.format(without_extension)
        else:
            ly_file_path = None
        result = type(self)(illustration).as_ly(ly_file_path, **kwargs)
        ly_file_path, abjad_formatting_time = result
        timer = systemtools.Timer()
        with timer:
            systemtools.IOManager.run_lilypond(ly_file_path)
        lilypond_rendering_time = timer.elapsed_time
        if os.name == 'nt':
            extension = 'mid'
        else:
            extension = 'midi'
        midi_file_path = '{}.{}'.format(
            os.path.splitext(ly_file_path)[0],
            extension,
            )
        if remove_ly:
            os.remove(ly_file_path)
        return midi_file_path, abjad_formatting_time, lilypond_rendering_time

    def as_module(self, module_file_path, object_name):
        r'''Persists client as Python module.

        ::

            >>> inventory = timespantools.TimespanInventory([
            ...     timespantools.Timespan(0, 1),
            ...     timespantools.Timespan(2, 4),
            ...     timespantools.Timespan(6, 8),
            ...     ])
            >>> persist(inventory).as_module( # doctest: +SKIP
            ...     '~/example.py',
            ...     'inventory',
            ...     )

        Returns none.
        '''
        from abjad.tools import systemtools
        assert '_storage_format_specification' in dir(self._client)
        result = ['# -*- encoding: utf-8 -*-']
        storage_pieces = format(self._client, 'storage').splitlines()
        pattern = re.compile(r'\b[a-z]+tools\b')
        tools_package_names = set()
        for line in storage_pieces:
            match = pattern.search(line)
            while match is not None:
                group = match.group()
                tools_package_names.add(group)
                end = match.end()
                match = pattern.search(line, pos=end)
        for name in sorted(tools_package_names):
            line = 'from abjad.tools import {}'.format(name)
            result.append(line)
        result.append('')
        result.append('')
        line = '{} = {}'.format(object_name, storage_pieces[0])
        result.append(line)
        result.extend(storage_pieces[1:])
        result = '\n'.join(result)
        module_file_path = os.path.expanduser(module_file_path)
        directory = os.path.dirname(module_file_path)
        systemtools.IOManager._ensure_directory_existence(directory)
        with open(module_file_path, 'w') as f:
            f.write(result)

    def as_pdf(
        self,
        pdf_file_path=None,
        candidacy=False,
        illustrate_function=None,
        remove_ly=False,
        **kwargs
        ):
        r'''Persists client as PDF.

        Autogenerates file path when `pdf_file_path` is none.

        ::

            >>> staff = Staff("c'4 e'4 d'4 f'4")
            >>> for x in persist(staff).as_pdf(): # doctest: +SKIP
            ...     x
            ...
            '/Users/josiah/.abjad/output/1416.pdf'
            0.047142982482910156
            0.7839350700378418

        When `candidacy` is true, writes PDF output only when PDF 
        output would differ from any existing output file.

        When `candidacy` is false, writes PDF output even if PDF
        output would not differ from any existing output file.

        Returns output path, elapsed formatting time and elapsed rendering
        time when PDF output is written.

        Returns false when when PDF output is not written due to 
        `candidacy` being set.
        '''
        from abjad.tools import systemtools
        if illustrate_function is None:
            assert '__illustrate__' in dir(self._client)
        if pdf_file_path is not None:
            pdf_file_path = os.path.expanduser(pdf_file_path)
            without_extension = os.path.splitext(pdf_file_path)[0]
            ly_file_path = '{}.ly'.format(without_extension)
        else:
            ly_file_path = None
        result = self.as_ly(
            ly_file_path,
            candidacy=candidacy,
            illustrate_function=illustrate_function,
            **kwargs
            )
        if candidacy and result == False:
            return False
        ly_file_path, abjad_formatting_time = result
        without_extension = os.path.splitext(ly_file_path)[0]
        pdf_file_path = '{}.pdf'.format(without_extension)
        timer = systemtools.Timer()
        with timer:
            success = systemtools.IOManager.run_lilypond(
                ly_file_path,
                candidacy=candidacy,
                )
        lilypond_rendering_time = timer.elapsed_time
        if remove_ly:
            os.remove(ly_file_path)
        if candidacy and not success:
            return False
        else:
            return (
                pdf_file_path,
                abjad_formatting_time,
                lilypond_rendering_time,
                success,
                )

    def as_png(
        self,
        png_file_path=None,
        remove_ly=False,
        illustrate_function=None,
        **kwargs
        ):
        r'''Persists client as PNG.

        ::

            >>> staff = Staff()
            >>> measure = Measure((4, 4), "c'4 d'4 e'4 f'4")
            >>> command = indicatortools.LilyPondCommand('break', 'after')
            >>> attach(command, measure[-1])
            >>> staff.extend(measure * 200)

        ::

            >>> result = persist(staff).as_png() # doctest: +SKIP
            >>> for x in result[0]: # doctest: +SKIP
            ...     x
            ...
            '/Users/josiah/Desktop/test-page1.png'
            '/Users/josiah/Desktop/test-page2.png'
            '/Users/josiah/Desktop/test-page3.png'
            '/Users/josiah/Desktop/test-page4.png'
            '/Users/josiah/Desktop/test-page5.png'
            '/Users/josiah/Desktop/test-page6.png'
            '/Users/josiah/Desktop/test-page7.png'
            '/Users/josiah/Desktop/test-page8.png'
            '/Users/josiah/Desktop/test-page9.png'
            '/Users/josiah/Desktop/test-page10.png'
            '/Users/josiah/Desktop/test-page11.png'
            '/Users/josiah/Desktop/test-page12.png'
            '/Users/josiah/Desktop/test-page13.png'
            '/Users/josiah/Desktop/test-page14.png'
            '/Users/josiah/Desktop/test-page15.png'

        Autogenerates file path when `png_file_path` is none.

        Returns output path(s), elapsed formatting time and elapsed rendering
        time.
        '''
        from abjad.tools import systemtools
        if illustrate_function is None:
            assert '__illustrate__' in dir(self._client)
        if png_file_path is not None:
            png_file_path = os.path.expanduser(png_file_path)
            without_extension = os.path.splitext(png_file_path)[0]
            ly_file_path = '{}.ly'.format(without_extension)
        else:
            ly_file_path = None
        result = self.as_ly(
            ly_file_path,
            illustrate_function=illustrate_function,
            **kwargs
            )

        ly_file_path, abjad_formatting_time = result

        original_directory = os.path.split(ly_file_path)[0]
        original_ly_file_path = ly_file_path
        temporary_directory = tempfile.mkdtemp()
        temporary_ly_file_path = os.path.join(
            temporary_directory,
            os.path.split(ly_file_path)[1],
            )
        shutil.copy(original_ly_file_path, temporary_ly_file_path)

        timer = systemtools.Timer()
        with timer:
            success = systemtools.IOManager.run_lilypond(
                temporary_ly_file_path,
                flags='--png',
                )
        lilypond_rendering_time = timer.elapsed_time

        png_file_paths = []
        for file_name in os.listdir(temporary_directory):
            if not file_name.endswith('.png'):
                continue
            source_png_file_path = os.path.join(
                temporary_directory,
                file_name,
                )
            target_png_file_path = os.path.join(
                original_directory,
                file_name,
                )
            shutil.move(source_png_file_path, target_png_file_path)
            png_file_paths.append(target_png_file_path)
        shutil.rmtree(temporary_directory)

        if remove_ly:
            os.remove(ly_file_path)

        if 1 < len(png_file_paths):
            png_file_paths.sort(
                key=lambda x: int(self._png_page_pattern.match(x).groups()[0]),
                )

        return (
            tuple(png_file_paths),
            abjad_formatting_time,
            lilypond_rendering_time,
            success,
            )


# -*- encoding: utf-8 -*-
from abjad import *


def test_datastructuretools_CyclicTree___eq___01():

    cyclic_tree_1 = datastructuretools.CyclicPayloadTree([[1, 2], [3, 4]])
    cyclic_tree_2 = datastructuretools.CyclicPayloadTree([[1, 2], [3, 4]])
    cyclic_tree_3 = datastructuretools.CyclicPayloadTree([[5, 6], [7, 8]])

    assert     cyclic_tree_1 == cyclic_tree_2
    assert not cyclic_tree_1 == cyclic_tree_3
    assert not cyclic_tree_2 == cyclic_tree_3


# -*- encoding: utf-8 -*-
from abjad.tools.abjadbooktools.OutputFormat import OutputFormat


class LaTeXOutputFormat(OutputFormat):
    r'''LaTeX output format.
    '''

    ### INITIALIZER ###

    def __init__(self):
        code_block_opening = '\\begin{lstlisting}'
        code_block_opening += '[basicstyle=\\footnotesize\\ttfamily, '
        code_block_opening += 'breaklines=True, tabsize=4, '
        code_block_opening += 'showtabs=false, showspaces=false]\n'
        code_block_closing = '\n\\end{lstlisting}\n'
        code_indent = 0
        image_block = '\\includegraphics[scale={scale}]'
        image_block += '{{images/{image_file_name}.pdf}}\n'
        image_format = 'pdf'
        OutputFormat.__init__(
            self,
            code_block_opening,
            code_block_closing,
            code_indent,
            image_block,
            image_format,
            )


# -*- encoding: utf-8 -*-


def list_all_experimental_classes(modules=None):
    r'''Lists all public classes defined in the Abjad experimental branch.

    ::

        >>> all_classes = documentationtools.list_all_experimental_classes()

    '''
    from abjad.tools import documentationtools
    return documentationtools.list_all_classes('experimental')


# -*- encoding: utf-8 -*-
from abjad.tools.abctools.AbjadObject import AbjadObject


class StatalServer(AbjadObject):
    r'''A statal server.
    '''

    ### INITIALIZER ###

    def __init__(self, cyclic_tree=None):
        from abjad.tools import datastructuretools
        self._cyclic_tree = datastructuretools.CyclicPayloadTree(cyclic_tree)

    ### SPECIAL METHODS ###

    def __call__(self, position=None, reverse=False):
        r'''Calls statal server.

        Returns statal server cursor.
        '''
        from abjad.tools import datastructuretools
        if isinstance(position, int):
            position = (position,)
        cursor = datastructuretools.StatalServerCursor(
            statal_server=self,
            position=position,
            reverse=reverse,
            )
        return cursor

    def __eq__(self, expr):
        r'''Is true when `expr` is a statal server with cyclic tree equal to
        that of this statal server. Otherwise false.

        Returns boolean.
        '''
        if isinstance(expr, type(self)):
            if self.cyclic_tree == expr.cyclic_tree:
                return True
        return False

    def __hash__(self):
        r'''Hashes statal server.

        Required to be explicitely re-defined on Python 3 if __eq__ changes.

        Returns integer.
        '''
        return super(StatalServer, self).__hash__()

    ### PUBLIC PROPERTIES ###

    @property
    def cyclic_tree(self):
        r'''Statal server cyclic tree.
        '''
        return self._cyclic_tree

    @property
    def last_node(self):
        r'''Statal server last node.
        '''
        return self.last_nodes[-1]


# -*- encoding: utf-8 -*-
from abjad.tools.abctools import AbjadObject


class LilyPondLanguageToken(AbjadObject):
    r'''A LilyPond file ``\language`` token.

    ..  container:: example

        ::

            >>> lilypondfiletools.LilyPondLanguageToken()
            LilyPondLanguageToken('english')

    '''

    ### CLASS VARIABLES ###

    __slots__ = (
        '_lilypond_language',
        )

    ### INITIALIZER ###

    def __init__(self):
        from abjad import abjad_configuration
        lilypond_language = abjad_configuration['lilypond_language']
        self._lilypond_language = lilypond_language

    ### SPECIAL METHODS ###

    def __format__(self, format_specification=''):
        r'''Formats LilyPond language token.

        ..  container:: example

            ::

                >>> token = lilypondfiletools.LilyPondLanguageToken()
                >>> print(format(token))
                \language "english"

        Returns string.
        '''
        from abjad.tools import systemtools
        if format_specification in ('', 'lilypond'):
            return self._lilypond_format
        elif format_specification == 'storage':
            return systemtools.StorageFormatManager.get_storage_format(self)
        return str(self)


    def __repr__(self):
        r'''Gets interpreter representation of LilyPond language token.

        ..  container:: example

            ::

                >>> token = lilypondfiletools.LilyPondLanguageToken()
                >>> token
                LilyPondLanguageToken('english')

        Returns string.
        '''
        return '{}({!r})'.format(
            type(self).__name__,
            self._lilypond_language,
            )

    ### PRIVATE PROPERTIES ###

    @property
    def _lilypond_format(self):
        lilypond_language = self._lilypond_language.lower()
        string = r'\language "{}"'.format(lilypond_language)
        return string



# -*- encoding: utf-8 -*-
from abjad import *


def test_rhythmmakertools_BurnishSpecifier_burnish_divisions_01():

    burnish_specifier = rhythmmakertools.BurnishSpecifier(
        burnish_divisions=True,
        lefts=(-1,),
        middles=(0,),
        rights=(-1,),
        left_lengths=(2,),
        right_lengths=(1,),
        )

    talea = rhythmmakertools.Talea(
        counts=(1, 1, 2, 4),
        denominator=32,
        )

    maker = rhythmmakertools.TaleaRhythmMaker(
        talea=talea,
        burnish_specifier=burnish_specifier,
        extra_counts_per_division=(0,),
        )

    divisions = [(5, 16), (6, 16)]
    music = maker(divisions)

    music = sequencetools.flatten_sequence(music)
    staff = Staff(scoretools.make_spacer_skip_measures(divisions))
    mutate(staff).replace_measure_contents(music)

    assert systemtools.TestManager.compare(
        staff,
        r'''
        \new Staff {
            {
                \time 5/16
                {
                    r32
                    r32
                    c'16 [
                    c'8
                    c'32 ]
                    r32
                }
            }
            {
                \time 6/16
                {
                    r16
                    r8
                    c'32 [
                    c'32
                    c'16 ]
                    r16
                }
            }
        }
        '''
        )


def test_rhythmmakertools_BurnishSpecifier_burnish_divisions_02():

    burnish_specifier = rhythmmakertools.BurnishSpecifier(
        burnish_divisions=True,
        lefts=(0,),
        middles=(-1,),
        rights=(0,),
        left_lengths=(2,),
        right_lengths=(1,),
        )

    talea = rhythmmakertools.Talea(
        counts=(1, 1, 2, 4),
        denominator=32,
        )

    maker = rhythmmakertools.TaleaRhythmMaker(
        talea=talea,
        extra_counts_per_division=(0,),
        burnish_specifier=burnish_specifier,
        )

    divisions = [(5, 16), (6, 16)]
    music = maker(divisions)

    music = sequencetools.flatten_sequence(music)
    staff = Staff(scoretools.make_spacer_skip_measures(divisions))
    mutate(staff).replace_measure_contents(music)

    assert systemtools.TestManager.compare(
        staff,
        r'''
        \new Staff {
            {
                \time 5/16
                {
                    c'32 [
                    c'32 ]
                    r16
                    r8
                    r32
                    c'32
                }
            }
            {
                \time 6/16
                {
                    c'16 [
                    c'8 ]
                    r32
                    r32
                    r16
                    c'16
                }
            }
        }
        '''
        )


def test_rhythmmakertools_BurnishSpecifier_burnish_divisions_03():

    burnish_specifier = rhythmmakertools.BurnishSpecifier(
        burnish_divisions=True,
        lefts=(0,),
        middles=(-1,),
        rights=(0,),
        left_lengths=(2,),
        right_lengths=(1,),
        )

    talea= rhythmmakertools.Talea(
        counts=(1, 1, 2, 4,),
        denominator=32,
        )

    maker = rhythmmakertools.TaleaRhythmMaker(
        talea=talea,
        extra_counts_per_division=(3,),
        burnish_specifier=burnish_specifier,
        )

    divisions = [(5, 16), (6, 16)]
    music = maker(divisions)

    music = sequencetools.flatten_sequence(music)
    staff = Staff(scoretools.make_spacer_skip_measures(divisions))
    mutate(staff).replace_measure_contents(music)

    assert systemtools.TestManager.compare(
        staff,
        r'''
        \new Staff {
            {
                \time 5/16
                \tweak #'text #tuplet-number::calc-fraction-text
                \times 10/13 {
                    c'32 [
                    c'32 ]
                    r16
                    r8
                    r32
                    r32
                    r16
                    c'32 ~
                }
            }
            {
                \time 6/16
                \times 4/5 {
                    c'16. [
                    c'32 ]
                    r32
                    r16
                    r8
                    r32
                    r32
                    c'16
                }
            }
        }
        '''
        )


def test_rhythmmakertools_BurnishSpecifier_burnish_divisions_04():

    burnish_specifier = rhythmmakertools.BurnishSpecifier(
        burnish_divisions=True,
        lefts=(-1,),
        middles=(0,),
        rights=(-1,),
        left_lengths=(1,),
        right_lengths=(1,),
        )

    talea = rhythmmakertools.Talea(
        counts=(1, 1, 2, 4),
        denominator=32,
        )

    maker = rhythmmakertools.TaleaRhythmMaker(
        talea=talea,
        extra_counts_per_division=(0, 3),
        burnish_specifier=burnish_specifier,
        )

    divisions = [(5, 16), (6, 16)]
    music = maker(divisions)

    music = sequencetools.flatten_sequence(music)
    staff = Staff(scoretools.make_spacer_skip_measures(divisions))
    mutate(staff).replace_measure_contents(music)

    assert systemtools.TestManager.compare(
        staff,
        r'''
        \new Staff {
            {
                \time 5/16
                {
                    r32
                    c'32 [
                    c'16
                    c'8
                    c'32 ]
                    r32
                }
            }
            {
                \time 6/16
                \times 4/5 {
                    r16
                    c'8 [
                    c'32
                    c'32
                    c'16
                    c'8 ]
                    r32
                }
            }
        }
        '''
        )


def test_rhythmmakertools_BurnishSpecifier_burnish_divisions_05():

    burnish_specifier = rhythmmakertools.BurnishSpecifier(
        burnish_divisions=True,
        lefts=(-1,),
        middles=(0,),
        rights=(-1,),
        left_lengths=(1,),
        right_lengths=(1,),
        )

    talea = rhythmmakertools.Talea(
        counts=(1, 1, 2, 4),
        denominator=32,
        )

    maker = rhythmmakertools.TaleaRhythmMaker(
        talea=talea,
        extra_counts_per_division=(0, 3),
        burnish_specifier=burnish_specifier,
        split_divisions_by_counts=(14,),
        )

    divisions = [(5, 16), (6, 16)]
    music = maker(divisions)

    music = sequencetools.flatten_sequence(music)
    staff = Staff(scoretools.make_spacer_skip_measures(divisions))
    mutate(staff).replace_measure_contents(music)

    assert systemtools.TestManager.compare(
        staff,
        r'''
        \new Staff {
            {
                \time 5/16
                {
                    r32
                    c'32 [
                    c'16
                    c'8
                    c'32 ]
                    r32
                }
            }
            {
                \time 6/16
                \times 4/7 {
                    r16
                    c'8
                    r32
                }
                {
                    r32
                    c'16 [
                    c'8 ]
                    r32
                }
            }
        }
        '''
        )


# -*- encoding: utf-8 -*-
import pytest
from abjad import *


def test_mathtools_integer_to_binary_string_01():
    r'''Returns base-2 representation of integer n as string.
    '''

    assert mathtools.integer_to_binary_string(1) == '1'
    assert mathtools.integer_to_binary_string(2) == '10'
    assert mathtools.integer_to_binary_string(3) == '11'
    assert mathtools.integer_to_binary_string(4) == '100'
    assert mathtools.integer_to_binary_string(5) == '101'
    assert mathtools.integer_to_binary_string(6) == '110'
    assert mathtools.integer_to_binary_string(7) == '111'
    assert mathtools.integer_to_binary_string(8) == '1000'


def test_mathtools_integer_to_binary_string_02():

    assert mathtools.integer_to_binary_string(0) == '0'
    assert mathtools.integer_to_binary_string(-1) == '-1'
    assert mathtools.integer_to_binary_string(-2) == '-10'
    assert mathtools.integer_to_binary_string(-3) == '-11'


def test_mathtools_integer_to_binary_string_03():
    r'''Raise TypeError for noninteger input.
    '''

    assert pytest.raises(TypeError, 'mathtools.integer_to_binary_string(5.5)')


# -*- encoding: utf-8 -*-
from abjad import *


def test_sequencetools_yield_all_permutations_of_sequence_01():
    r'''Yield all permtuations of tuple.
    '''

    sequence = (1, 2, 3)
    generator = sequencetools.yield_all_permutations_of_sequence(sequence)
    permutations = list(generator)
    assert permutations == [(1, 2, 3), (1, 3, 2), (2, 1, 3), (2, 3, 1), (3, 1, 2), (3, 2, 1)]


# TODO: remove this functionality
def test_sequencetools_yield_all_permutations_of_sequence_02():
    r'''Yield all permtuations of Abjad container.
    '''

    container = Container("c'8 d'8 e'8")
    generator = sequencetools.yield_all_permutations_of_sequence(container)
    permutations = list(generator)
    assert str(permutations) == '[Container("c\'8 d\'8 e\'8"), Container("c\'8 e\'8 d\'8"), Container("d\'8 c\'8 e\'8"), Container("d\'8 e\'8 c\'8"), Container("e\'8 c\'8 d\'8"), Container("e\'8 d\'8 c\'8")]'


# -*- encoding: utf-8 -*-


def remove_repeated_elements(sequence):
    r'''Removes repeated elements from `sequence`.

    ..  container:: example

        ::

            >>> sequence = (17, 18, 18, 18, 19, 20, 21, 22, 22, 24, 24)
            >>> sequencetools.remove_repeated_elements(sequence)
            (17, 18, 19, 20, 21, 22, 24)

    ..  container:: example

        ::

            >>> sequence = [31, 31, 35, 35, 31, 31, 31, 31, 35]
            >>> sequencetools.remove_repeated_elements(sequence)
            [31, 35, 31, 35]

    Returns new object of `sequence` type.
    '''

    if not sequence:
        return type(sequence)()

    result = [sequence[0]]
    for element in sequence[1:]:
        if element != result[-1]:
            result.append(element)

    result = type(sequence)(result)
    return result


# -*- encoding: utf-8 -*-


def to_upper_camel_case(string):
    r'''Changes `string` to upper camel case.

    ..  container:: example

        Changes words to upper camel case:

        ::

            >>> stringtools.to_upper_camel_case('scale degrees 4 and 5')
            'ScaleDegrees4And5'

    ..  container:: example

        Changes snake case to upper camel case:

        ::

            >>> stringtools.to_upper_camel_case('scale_degrees_4_and_5')
            'ScaleDegrees4And5'

    ..  container:: example

        Changes dash case to upper camel case:

        ::

            >>> stringtools.to_upper_camel_case('scale-degrees-4-and-5')
            'ScaleDegrees4And5'

    ..  container:: example

        Changes upper camel case to upper camel case:

        ::

            >>> stringtools.to_upper_camel_case('ScaleDegrees4And5')
            'ScaleDegrees4And5'

    Returns string.
    '''
    from abjad.tools import stringtools

    words = stringtools.delimit_words(string)
    words = [_.capitalize() for _ in words]
    result = ''.join(words)

    return result


# -*- encoding: utf-8 -*-
from abjad.tools import mathtools
from abjad.tools.abctools.AbjadObject import AbjadObject


class StemTremolo(AbjadObject):
    '''A stem tremolo.

    ::

        >>> note = Note("c'4")
        >>> stem_tremolo = indicatortools.StemTremolo(16)
        >>> attach(stem_tremolo, note)
        >>> show(note) # doctest: +SKIP

    ..  doctest::

        >>> print(format(note))
        c'4 :16

    '''

    ### CLASS VARIABLES ###

    __slots__ = (
        '_tremolo_flags',
        )

    _format_slot = 'right'

    ### INITIALIZER ###

    def __init__(self, *args):
        if len(args) == 1 and isinstance(args[0], type(self)):
            tremolo_flags = args[0].tremolo_flags
        elif len(args) == 1 and not isinstance(args[0], type(self)):
            tremolo_flags = args[0]
        elif len(args) == 0:
            tremolo_flags = 16
        else:
            message = 'can not initialize {}: {!r}.'
            message = message.format(type(self).__name__, args)
            raise ValueError(message)
        if not mathtools.is_nonnegative_integer_power_of_two(tremolo_flags):
            message = 'must be nonnegative integer power of 2: {!r}.'
            message = message.format(tremolo_flags)
            raise ValueError(message)
        self._tremolo_flags = tremolo_flags

    ### SPECIAL METHODS ###

    def __copy__(self, *args):
        r'''Copies stem tremolo.

        ::

            >>> import copy
            >>> stem_tremolo_1 = indicatortools.StemTremolo(16)
            >>> stem_tremolo_2 = copy.copy(stem_tremolo_1)

        ::

            >>> stem_tremolo_1 == stem_tremolo_2
            True

        ::

            >>> stem_tremolo_1 is not stem_tremolo_2
            True

        Returns new stem tremolo.
        '''
        return type(self)(self.tremolo_flags)

    def __eq__(self, expr):
        r'''Is true when `expr` is a stem tremolo with a tremolo flag count
        equal to that of this stem tremolo.  Otherwise false:

        ::

            >>> stem_tremolo_1 = indicatortools.StemTremolo(16)
            >>> stem_tremolo_2 = indicatortools.StemTremolo(16)
            >>> stem_tremolo_3 = indicatortools.StemTremolo(32)

        ::

            >>> stem_tremolo_1 == stem_tremolo_1
            True
            >>> stem_tremolo_1 == stem_tremolo_2
            True
            >>> stem_tremolo_1 == stem_tremolo_3
            False
            >>> stem_tremolo_2 == stem_tremolo_1
            True
            >>> stem_tremolo_2 == stem_tremolo_2
            True
            >>> stem_tremolo_2 == stem_tremolo_3
            False
            >>> stem_tremolo_3 == stem_tremolo_1
            False
            >>> stem_tremolo_3 == stem_tremolo_2
            False
            >>> stem_tremolo_3 == stem_tremolo_3
            True

        Returns boolean.
        '''
        if isinstance(expr, type(self)):
            if self.tremolo_flags == expr.tremolo_flags:
                return True
        return False

    def __format__(self, format_specification=''):
        r'''Formats stem tremolo.

        ::

            >>> print(format(stem_tremolo))
            :16

        Returns string.
        '''
        from abjad.tools import systemtools
        if format_specification in ('', 'lilypond'):
            return self._lilypond_format
        elif format_specification == 'storage':
            return systemtools.StorageFormatManager.get_storage_format(self)
        return str(self)

    def __hash__(self):
        r'''Hashes stem tremolo.

        Required to be explicitely re-defined on Python 3 if __eq__ changes.

        Returns integer.
        '''
        return super(StemTremolo, self).__hash__()

    def __str__(self):
        r'''String representation of stem tremolo.

        Returns string.
        '''
        return ':{!s}'.format(self.tremolo_flags)

    ### PRIVATE PROPERTIES ###

    @property
    def _lilypond_format(self):
        return str(self)

    @property
    def _lilypond_format_bundle(self):
        from abjad.tools import systemtools
        lilypond_format_bundle = systemtools.LilyPondFormatBundle()
        lilypond_format_bundle.right.stem_tremolos.append(str(self))
        return lilypond_format_bundle

    @property
    def _storage_format_specification(self):
        from abjad.tools import systemtools
        return systemtools.StorageFormatSpecification(
            self,
            is_indented=False,
            positional_argument_values=(
                self.tremolo_flags,
                ),
            )

    ### PUBLIC PROPERTIES ###

    @property
    def tremolo_flags(self):
        r'''Flags of stem tremolo.

        ::

            >>> stem_tremolo.tremolo_flags
            16

        Returns nonnegative integer power of ``2``.
        '''
        return self._tremolo_flags



# -*- encoding: utf-8 -*-
import os


def show(expr, return_timing=False, **kwargs):
    r'''Shows `expr`.

    ..  container:: example

        Shows a note:

        ::

            >>> note = Note("c'4")
            >>> show(note) # doctest: +SKIP

    Abjad writes LilyPond input files to the ``~/.abjad/output/``
    directory by default.

    You may change this by setting the ``abjad_output_directory`` variable in
    the Abjad ``config.py`` file.

    Returns none when `return_timing` is false.

    Returns pair of `abjad_formatting_time` and `lilypond_rendering_time`
    when `return_timing` is true.
    '''
    from abjad.tools import systemtools
    from abjad.tools import topleveltools
    assert '__illustrate__' in dir(expr)
    result = topleveltools.persist(expr).as_pdf(**kwargs)
    pdf_file_path = result[0]
    abjad_formatting_time = result[1]
    lilypond_rendering_time = result[2]
    success = result[3]
    if success:
        systemtools.IOManager.open_file(pdf_file_path)
    if return_timing:
        return abjad_formatting_time, lilypond_rendering_time


# -*- encoding: utf-8 -*-
from abjad import *
from abjad.demos.mozart.make_mozart_score import make_mozart_score


def make_mozart_lilypond_file():
    r'''Makes Mozart LilyPond file.
    '''

    score = make_mozart_score()
    lily = lilypondfiletools.make_basic_lilypond_file(score)
    title = markuptools.Markup(r'\bold \sans "Ein Musikalisches Wuerfelspiel"')
    composer = schemetools.Scheme("W. A. Mozart (maybe?)")
    lily.global_staff_size = 12
    lily.header_block.title = title
    lily.header_block.composer = composer
    lily.layout_block.ragged_right = True
    lily.paper_block.markup_system_spacing__basic_distance = 8
    lily.paper_block.paper_width = 180
    return lily


# -*- encoding: utf-8 -*-
from abjad.tools import mathtools
from abjad.tools.abctools.AbjadObject import AbjadObject


# TODO: remove multiple inheritance; hold a private _tuple instead
class CyclicTuple(AbjadObject, tuple):
    '''A cylic tuple.

    ::

        >>> cyclic_tuple = datastructuretools.CyclicTuple('abcd')

    ::

        >>> cyclic_tuple
        CyclicTuple(['a', 'b', 'c', 'd'])

    ::

        >>> for x in range(8):
        ...     print(x, cyclic_tuple[x])
        ... 
        0 a
        1 b
        2 c
        3 d
        4 a
        5 b
        6 c
        7 d

    Cyclic tuples overload the item-getting method of built-in tuples.

    Cyclic tuples return a value for any integer index.

    Cyclic tuples otherwise behave exactly like built-in tuples.
    '''

    ### SPECIAL METHODS ###

    def __eq__(self, expr):
        r'''Is true when `expr` is a tuple with items equal to those of this
        cyclic tuple. Otherwise false.

        Returns boolean.
        '''
        return tuple.__eq__(self, expr)

    def __getitem__(self, i):
        r'''Gets `i` from cyclic tuple.

        Raises index error when `i` can not be found in cyclic tuple.

        Returns item.
        '''
        if isinstance(i, slice):
            return self.__getslice__(i.start, i.stop)
        if not self:
            raise IndexError
        i = i % len(self)
        return tuple.__getitem__(self, i)

    def __getslice__(self, start_index, stop_index):
        r'''Gets slice of items from `start_index` to `stop_index` in cyclic
        tuple.

        ..  container:: example

            Gets slice open at right:

            ::

                >>> sequence = [0, 1, 2, 3, 4, 5]
                >>> sequence = datastructuretools.CyclicTuple(sequence)
                >>> sequence[2:]
                (2, 3, 4, 5)

        ..  container:: example

            Gets slice closed at right:

            ::

                >>> sequence = [0, 1, 2, 3, 4, 5]
                >>> sequence = datastructuretools.CyclicTuple(sequence)
                >>> sequence[:15]
                (0, 1, 2, 3, 4, 5, 0, 1, 2, 3, 4, 5, 0, 1, 2)

        Returns tuple.
        '''
        if stop_index is not None and 1000000 < stop_index:
            stop_index = len(self)
        result = []
        if start_index is None:
            start_index = 0
        if stop_index is None:
            indices = range(start_index, len(self))
        else:
            indices = range(start_index, stop_index)
        result = [self[n] for n in indices]
        return tuple(result)

    def __hash__(self):
        r'''Hashes cyclic tuple.

        Required to be explicitely re-defined on Python 3 if __eq__ changes.

        Returns integer.
        '''
        return super(CyclicTuple, self).__hash__()

    def __str__(self):
        r'''String representation of cyclic tuple.

        Returns string.
        '''
        if self:
            contents = [str(x) for x in self]
            contents = ', '.join(contents)
            string = '[{!s}]'.format(contents)
            return string
        return '()'

    ### PRIVATE PROPERTIES ###

    @property
    def _storage_format_specification(self):
        from abjad.tools import systemtools
        return systemtools.StorageFormatSpecification(
            self,
            positional_argument_values=(
                list(self),
                ),
            )



# -*- encoding: utf-8 -*-
from abjad import *


def test_markuptools_make_vertically_adjusted_composer_markup_01():

    markup = markuptools.make_vertically_adjusted_composer_markup(
        'Josquin Desprez')

    assert systemtools.TestManager.compare(
        format(markup, 'lilypond'),
        r'''
        \markup {
            \override
                #'(font-name . "Times")
                {
                    \hspace
                        #0
                    \raise
                        #-20
                        \fontsize
                            #3
                            "Josquin Desprez"
                    \hspace
                        #0
                }
            }
        ''',
        )


# -*- encoding: utf-8 -*-
import collections
from abjad.tools.abctools import AbjadObject
from abjad.tools.topleveltools import inspect_
from abjad.tools.topleveltools import iterate


class ContextMap(AbjadObject):
    r'''A context map.

    ::

        >>> template = templatetools.StringOrchestraScoreTemplate()
        >>> context_map = datastructuretools.ContextMap(template)

    ::

        >>> context_map['String Orchestra Score']['color'] = 'red'
        >>> context_map['Violin Staff Group']['color'] = 'blue'
        >>> context_map['Contrabass Staff Group']['color'] = 'green'
        >>> context_map['Contrabass 1 Voice']['color'] = 'yellow'

    ::

        >>> print(format(context_map))
        datastructuretools.ContextMap(
            score_template=templatetools.StringOrchestraScoreTemplate(
                violin_count=6,
                viola_count=4,
                cello_count=3,
                contrabass_count=2,
                ),
            settings=[
                (
                    'Contrabass 1 Voice',
                    [
                        ('color', 'yellow'),
                        ],
                    ),
                (
                    'Contrabass Staff Group',
                    [
                        ('color', 'green'),
                        ],
                    ),
                (
                    'String Orchestra Score',
                    [
                        ('color', 'red'),
                        ],
                    ),
                (
                    'Violin Staff Group',
                    [
                        ('color', 'blue'),
                        ],
                    ),
                ],
            )

    ::

        >>> context_map['Violin 1 Voice']['color']
        'blue'

    ::

        >>> context_map['Viola 3 Voice']['color']
        'red'

    ::

        >>> context_map['Contrabass 1 Voice']['color']
        'yellow'

    ::

        >>> context_map['Contrabass 2 Voice']['color']
        'green'

    '''

    ### CLASS VARIABLES ###

    __slots__ = (
        '_components',
        '_score',
        '_score_template',
        )

    ### INITIALIZER ###

    def __init__(self, score_template=None, settings=None):
        from abjad.tools import scoretools
        from abjad.tools import templatetools
        if score_template is None:
            score_template = templatetools.StringOrchestraScoreTemplate()
        score = score_template()
        assert isinstance(score, scoretools.Score), repr(score)
        self._score = score
        self._score_template = score_template
        self._components = {}
        for context in iterate(self._score).by_class(scoretools.Context):
            assert context.name is not None, context.name
            component = _ContextMapComponent(
                self,
                context.name,
                )
            self._components[context.name] = component
        assert isinstance(settings, (dict, type(None)))
        if isinstance(settings, dict):
            for context_name, context_settings in settings.items():
                for key, value in context_settings.items():
                    self[context_name][key] = value

    ### SPECIAL METHODS ###

    def __getitem__(self, context_name):
        r'''Gets context map component for `context_name`.

        Returns context map component.
        '''
        from abjad.tools import scoretools
        if isinstance(context_name, scoretools.Context):
            context_name = context_name.name
        return self._components[context_name]

    ### PUBLIC METHODS ###

    def copy(self):
        r'''Copy context map.

        ::

            >>> copied_context_map = context_map.copy()
            >>> print(format(copied_context_map))
            datastructuretools.ContextMap(
                score_template=templatetools.StringOrchestraScoreTemplate(
                    violin_count=6,
                    viola_count=4,
                    cello_count=3,
                    contrabass_count=2,
                    ),
                settings=[
                    (
                        'Contrabass 1 Voice',
                        [
                            ('color', 'yellow'),
                            ],
                        ),
                    (
                        'Contrabass Staff Group',
                        [
                            ('color', 'green'),
                            ],
                        ),
                    (
                        'String Orchestra Score',
                        [
                            ('color', 'red'),
                            ],
                        ),
                    (
                        'Violin Staff Group',
                        [
                            ('color', 'blue'),
                            ],
                        ),
                    ],
                )

        '''
        score_template = self.score_template
        settings = self.settings
        copied = type(self)(
            score_template=score_template,
            settings=settings,
            )
        return copied

    def update(self, expr):
        r'''Updates context map with information in context map `expr`.

        ::

            >>> template = templatetools.StringQuartetScoreTemplate()
            >>> context_map_one = datastructuretools.ContextMap(template)
            >>> context_map_one['Cello Voice']['one'] = 1
            >>> context_map_one['Viola Voice']['foo'] = 'bar'
            >>> context_map_two = datastructuretools.ContextMap(template)
            >>> context_map_two['Cello Voice']['one'] = 'a'
            >>> context_map_two['String Quartet Score']['baz'] = (1, 2, 3)

        ::

            >>> print(format(context_map_one))
            datastructuretools.ContextMap(
                score_template=templatetools.StringQuartetScoreTemplate(),
                settings=[
                    (
                        'Cello Voice',
                        [
                            ('one', 1),
                            ],
                        ),
                    (
                        'Viola Voice',
                        [
                            ('foo', 'bar'),
                            ],
                        ),
                    ],
                )

        ::

            >>> print(format(context_map_two))
            datastructuretools.ContextMap(
                score_template=templatetools.StringQuartetScoreTemplate(),
                settings=[
                    (
                        'Cello Voice',
                        [
                            ('one', 'a'),
                            ],
                        ),
                    (
                        'String Quartet Score',
                        [
                            (
                                'baz',
                                (1, 2, 3),
                                ),
                            ],
                        ),
                    ],
                )

        ::

            >>> context_map_one.update(context_map_two)
            >>> print(format(context_map_one))
            datastructuretools.ContextMap(
                score_template=templatetools.StringQuartetScoreTemplate(),
                settings=[
                    (
                        'Cello Voice',
                        [
                            ('one', 'a'),
                            ],
                        ),
                    (
                        'String Quartet Score',
                        [
                            (
                                'baz',
                                (1, 2, 3),
                                ),
                            ],
                        ),
                    (
                        'Viola Voice',
                        [
                            ('foo', 'bar'),
                            ],
                        ),
                    ],
                )


        Operates in place and returns none.
        '''
        assert isinstance(expr, type(self))
        assert self.score_template == expr.score_template
        for context_name in self._components:
            self_component = self._components[context_name]
            expr_component = expr._components[context_name]
            for key, value in expr_component._context_settings.items():
                self_component[key] = value

    ### PUBLIC PROPERTIES ###

    @property
    def score_template(self):
        r'''Score template on which context map is based.

        ..  container:: example

            ::

                >>> print(format(context_map.score_template))
                templatetools.StringOrchestraScoreTemplate(
                    violin_count=6,
                    viola_count=4,
                    cello_count=3,
                    contrabass_count=2,
                    )

        Returns score template or none.
        '''
        return self._score_template

    @property
    def settings(self):
        r'''All settings for context map.

        Returns ordered dict or none.
        '''
        settings = collections.OrderedDict()
        for context_name in sorted(self._components):
            component = self._components[context_name]
            if not len(component):
                continue
            context_settings = collections.OrderedDict()
            for key in sorted(component._context_settings):
                context_settings[key] = component._context_settings[key]
            if context_settings:
                settings[context_name] = context_settings
        if not settings:
            return None
        return settings


class _ContextMapComponent(AbjadObject):
    r'''A component in a context map.
    '''

    ### CLASS VARIABLES ###

    __slots__ = (
        '_client',
        '_context_name',
        '_context_settings',
        )

    ### INITIALIZER ###

    def __init__(self, context_map, context_name):
        from abjad.tools import datastructuretools
        assert isinstance(context_map, datastructuretools.ContextMap)
        assert isinstance(context_name, str) and context_name
        self._client = context_map
        self._context_name = context_name
        self._context_settings = {}

    ### PRIVATE METHODS ###

    def _as_chain_map(self):
        dictionaries = []
        components = self._client._components
        score = self._client._score
        if self._context_name == score.name:
            context = score
        else:
            context = score[self._context_name]
        parentage = inspect_(context).get_parentage()
        for context in parentage:
            context_name = context.name
            context_settings = components[context_name]._context_settings
            dictionaries.append(context_settings)
        dictionaries.reverse()
        chain_map = dictionaries[0].copy()
        for dictionary in dictionaries[1:]:
            chain_map.update(dictionary)
        return chain_map

    ### PRIVATE PROPERTIES ###

    @property
    def _storage_format_specification(self):
        from abjad.tools import systemtools
        return systemtools.StorageFormatSpecification(
            self,
            positional_argument_values=(
                self._context_name,
                )
            )

    ### SPECIAL METHODS ###

    def __contains__(self, key):
        r'''Is true if context map component contains `key`. Otherwise
        false.

        Returns boolean.
        '''
        return self._as_chain_map().__contains__(key)

    def __getitem__(self, key):
        r'''Gets `key` for context map component.

        Returns value of `key`.
        '''
        assert isinstance(key, str)
        return self._as_chain_map().__getitem__(key)

    def __iter__(self):
        r'''Iterates over keys in context map component.

        Returns generator.
        '''
        return self._as_chain_map().__iter__()

    def __len__(self):
        r'''Gets number of keys reachable from context map component.

        Returns int.
        '''
        return len(self._as_chain_map())

    def __setitem__(self, key, value):
        r'''Sets `key` to `value` in context map component.

        Returns none.
        '''
        assert isinstance(key, str)
        self._context_settings[key] = value



# -*- encoding: utf-8 -*-
from abjad.tools.abctools import AbjadValueObject


class BeamSpecifier(AbjadValueObject):
    r'''Beam specifier.

    ..  container:: example

        Beam notes in each cell together but do not beam between cells:

        ::

            >>> specifier = rhythmmakertools.BeamSpecifier(
            ...     beam_each_division=True,
            ...     beam_divisions_together=False
            ...     )

    ..  container:: example

        Beam everything:

        ::

            >>> specifier = rhythmmakertools.BeamSpecifier(
            ...     beam_each_division=True,
            ...     beam_divisions_together=True
            ...     )

    ..  container:: example

        Beam nothing:

        ::

            >>> specifier = rhythmmakertools.BeamSpecifier(
            ...     beam_each_division=True,
            ...     beam_divisions_together=True
            ...     )

    Beam specifiers are immutable.
    '''

    ### CLASS VARIABLES ###

    __slots__ = (
        '_beam_divisions_together',
        '_beam_each_division',
        )

    ### INITIALIZER ###

    def __init__(
        self,
        beam_each_division=True,
        beam_divisions_together=False,
        ):
        assert isinstance(beam_each_division, bool)
        assert isinstance(beam_divisions_together, bool)
        self._beam_each_division = beam_each_division
        self._beam_divisions_together = beam_divisions_together

    ### SPECIAL METHODS ###

    def __eq__(self, arg):
        r'''Is true when `arg` is a beam specifier with `beam_each_division`
        and `beam_divisions_together` equal to those of this beam specifier.
        Otherwise false.

        ..  container:: example

            ::

                >>> specifier_1 = rhythmmakertools.BeamSpecifier(
                ...     beam_each_division=True,
                ...     )
                >>> specifier_2 = rhythmmakertools.BeamSpecifier(
                ...     beam_each_division=False,
                ...     )

            ::

                >>> specifier_1 == specifier_1
                True
                >>> specifier_1 == specifier_2
                False
                >>> specifier_2 == specifier_1
                False
                >>> specifier_2 == specifier_2
                True

        Returns boolean.
        '''
        if isinstance(arg, type(self)):
            if self.beam_each_division == arg.beam_each_division and \
                self.beam_divisions_together == arg.beam_divisions_together:
                return True
        return False

    def __format__(self, format_specification=''):
        r'''Formats beam specifier.

        ..  container:: example

            ::

                >>> specifier = rhythmmakertools.BeamSpecifier()
                >>> print(format(specifier))
                rhythmmakertools.BeamSpecifier(
                    beam_each_division=True,
                    beam_divisions_together=False,
                    )

        Returns string.
        '''
        return AbjadValueObject.__format__(
            self,
            format_specification=format_specification,
            )

    def __hash__(self):
        r'''Hashes beam specifier.

        Required to be explicitely re-defined on Python 3 if __eq__ changes.

        Returns integer.
        '''
        return super(BeamSpecifier, self).__hash__()

    def __repr__(self):
        r'''Gets interpreter representation of beam specifier.

        ..  container:: example

            ::

                >>> rhythmmakertools.BeamSpecifier()
                BeamSpecifier(beam_each_division=True, beam_divisions_together=False)

        Returns string.
        '''
        return AbjadValueObject.__repr__(self)

    ### PRIVATE PROPERTIES ###

    @property
    def _attribute_manifest(self):
        from abjad.tools import systemtools
        from scoremanager import idetools
        return systemtools.AttributeManifest(
            systemtools.AttributeDetail(
                name='beam_divisions_together',
                command='bdt',
                editor=idetools.getters.get_boolean,
                ),
            systemtools.AttributeDetail(
                name='beam_each_division',
                command='bed',
                editor=idetools.getters.get_boolean,
                ),
            )

    ### PUBLIC PROPERTIES ###

    @property
    def beam_divisions_together(self):
        r'''Is true when target should beam cells together.
        Otherwise false.

        ..  container:: example

            ::

                >>> specifier = rhythmmakertools.BeamSpecifier()
                >>> specifier.beam_divisions_together
                False

        Defaults to false.

        Returns boolean.
        '''
        return self._beam_divisions_together


    @property
    def beam_each_division(self):
        r'''Is true when target should beam each cell.
        Otherwise false.

        ..  container:: example

            ::

                >>> specifier = rhythmmakertools.BeamSpecifier()
                >>> specifier.beam_each_division
                True

        Defaults to true.

        Returns boolean.
        '''
        return self._beam_each_division


# -*- encoding: utf-8 -*-
import copy
from abjad.tools import datastructuretools
from abjad.tools import durationtools
from abjad.tools import mathtools
from abjad.tools import scoretools
from abjad.tools import selectiontools
from abjad.tools import sequencetools
from abjad.tools import spannertools
from abjad.tools.rhythmmakertools.RhythmMaker import RhythmMaker
from abjad.tools.topleveltools import attach
from abjad.tools.topleveltools import new


class IncisedRhythmMaker(RhythmMaker):
    r'''Incised rhythm-maker.

    ..  container:: example

        ::

            >>> incise_specifier = rhythmmakertools.InciseSpecifier(
            ...     incise_divisions=True,
            ...     prefix_talea=(-1,),
            ...     prefix_lengths=(0, 1),
            ...     suffix_talea=(-1,),
            ...     suffix_lengths=(1,),
            ...     talea_denominator=16,
            ...     )
            >>> maker = rhythmmakertools.IncisedRhythmMaker(
            ...     incise_specifier=incise_specifier,
            ...     )

        ::

            >>> divisions = 4 * [(5, 16)]
            >>> music = maker(divisions)
            >>> lilypond_file = rhythmmakertools.make_lilypond_file(
            ...     music,
            ...     divisions,
            ...     )
            >>> show(lilypond_file) # doctest: +SKIP

        ..  doctest::

            >>> staff = maker._get_rhythmic_staff(lilypond_file)
            >>> f(staff)
            \new RhythmicStaff {
                {
                    \time 5/16
                    c'4
                    r16
                }
                {
                    r16
                    c'8.
                    r16
                }
                {
                    c'4
                    r16
                }
                {
                    r16
                    c'8.
                    r16
                }
            }

    '''

    ### CLASS VARIABLES ###

    __slots__ = (
        '_extra_counts_per_division',
        '_helper_functions',
        '_incise_specifier',
        '_split_divisions_by_counts',
        )

    _class_name_abbreviatio = 'In'

    _human_readable_class_name = 'incised rhythm-maker'

    ### INITIALIZER ###

    def __init__(
        self,
        incise_specifier=None,
        split_divisions_by_counts=None,
        extra_counts_per_division=None,
        beam_specifier=None,
        duration_spelling_specifier=None,
        tie_specifier=None,
        helper_functions=None,
        ):
        from abjad.tools import rhythmmakertools
        RhythmMaker.__init__(
            self,
            beam_specifier=beam_specifier,
            duration_spelling_specifier=duration_spelling_specifier,
            tie_specifier=tie_specifier,
            )
        prototype = (rhythmmakertools.InciseSpecifier, type(None))
        assert isinstance(incise_specifier, prototype)
        self._incise_specifier = incise_specifier
        extra_counts_per_division = \
            self._to_tuple(extra_counts_per_division)
        split_divisions_by_counts = \
            self._to_tuple(split_divisions_by_counts)
        assert extra_counts_per_division is None or \
            mathtools.all_are_nonnegative_integer_equivalent_numbers(
            extra_counts_per_division), extra_counts_per_division
        assert split_divisions_by_counts is None or \
            mathtools.all_are_nonnegative_integer_equivalent_numbers(
            split_divisions_by_counts), split_divisions_by_counts
        self._extra_counts_per_division = extra_counts_per_division
        self._split_divisions_by_counts = split_divisions_by_counts
        if helper_functions is not None:
            assert isinstance(helper_functions, dict)
            for name in helper_functions:
                function = helper_functions.get(name)
                assert callable(function)
        self._helper_functions = helper_functions

    ### SPECIAL METHODS ###

    def __call__(self, divisions, seeds=None):
        r'''Calls incised rhythm-maker on `divisions`.

        Returns list of selections.
        '''
        return RhythmMaker.__call__(
            self,
            divisions,
            seeds=seeds,
            )

    ### PRIVATE METHODS ###

    def _make_division_incised_numeric_map(
        self,
        duration_pairs=None,
        prefix_talea=None,
        prefix_lengths=None,
        suffix_talea=None,
        suffix_lengths=None,
        extra_counts_per_division=None,
        ):
        numeric_map, prefix_talea_index, suffix_talea_index = [], 0, 0
        for pair_index, duration_pair in enumerate(duration_pairs):
            prefix_length, suffix_length = \
                prefix_lengths[pair_index], suffix_lengths[pair_index]
            prefix = prefix_talea[
                prefix_talea_index:prefix_talea_index+prefix_length]
            suffix = suffix_talea[
                suffix_talea_index:suffix_talea_index+suffix_length]
            prefix_talea_index += prefix_length
            suffix_talea_index += suffix_length
            prolation_addendum = extra_counts_per_division[pair_index]
            if isinstance(duration_pair, tuple):
                numerator = duration_pair[0] + (
                    prolation_addendum % duration_pair[0])
            else:
                numerator = duration_pair.numerator + (
                    prolation_addendum % duration_pair.numerator)
            numeric_map_part = self._make_numeric_map_part(
                numerator, prefix, suffix)
            numeric_map.append(numeric_map_part)
        return numeric_map

    def _make_middle_of_numeric_map_part(self, middle):
        from abjad.tools import rhythmmakertools
        incise_specifier = self.incise_specifier
        if incise_specifier is None:
            incise_specifier = rhythmmakertools.InciseSpecifier()
        if incise_specifier.fill_with_notes:
            if incise_specifier.incise_divisions:
                if 0 < middle:
                    if incise_specifier.body_ratio is not None:
                        shards = mathtools.divide_number_by_ratio(
                            middle, incise_specifier.body_ratio)
                        return tuple(shards)
                    else:
                        return (middle,)
                else:
                    return ()
            elif incise_specifier.incise_output:
                if 0 < middle:
                    return (middle,)
                else:
                    return ()
            else:
                message = 'must incise divisions or output.'
                raise Exception(message)
        else:
            if incise_specifier.incise_divisions:
                if 0 < middle:
                    return (-abs(middle),)
                else:
                    return ()
            elif incise_specifier.incise_output:
                if 0 < middle:
                    return (-abs(middle),)
                else:
                    return ()
            else:
                message = 'must incise divisions or output.'
                raise Exception(message)

    def _make_music(self, duration_pairs, seeds):
        from abjad.tools import rhythmmakertools
        input_ = self._prepare_input(seeds)
        prefix_talea = input_[0]
        prefix_lengths = input_[1]
        suffix_talea = input_[2]
        suffix_lengths = input_[3]
        extra_counts_per_division = input_[4]
        split_divisions_by_counts = input_[5]
        taleas = (
            prefix_talea,
            suffix_talea,
            extra_counts_per_division,
            split_divisions_by_counts,
            )
        input_ = self._scale_taleas(
            duration_pairs,
            self.incise_specifier.talea_denominator,
            taleas,
            )
        duration_pairs = input_[0]
        lcd = input_[1]
        prefix_talea = input_[2]
        suffix_talea = input_[3]
        extra_counts_per_division = input_[4]
        split_divisions_by_counts = input_[5]
        secondary_duration_pairs = self._make_secondary_duration_pairs(
            duration_pairs, split_divisions_by_counts)
        incise_specifier = self.incise_specifier
        if incise_specifier is None:
            incise_specifier = rhythmmakertools.InciseSpecifier()
        if incise_specifier.incise_divisions:
            numeric_map = self._make_division_incised_numeric_map(
                secondary_duration_pairs,
                prefix_talea,
                prefix_lengths,
                suffix_talea,
                suffix_lengths,
                extra_counts_per_division,
                )
        else:
            assert incise_specifier.incise_output
            numeric_map = self._make_output_incised_numeric_map(
                secondary_duration_pairs,
                prefix_talea,
                prefix_lengths,
                suffix_talea,
                suffix_lengths,
                extra_counts_per_division,
                )
        result = []
        selections = \
            self._numeric_map_and_talea_denominator_to_leaf_selections(
            numeric_map, lcd)
        if not self.extra_counts_per_division:
            result.extend(selections)
        else:
            tuplets = self._make_tuplets(
                secondary_duration_pairs,
                selections,
                )
            result.extend(tuplets)
        assert self._all_are_tuplets_or_all_are_leaf_selections(result)
        beam_specifier = self.beam_specifier
        if beam_specifier is None:
            beam_specifier = rhythmmakertools.BeamSpecifier()
        if beam_specifier.beam_divisions_together:
            beam = spannertools.MultipartBeam()
            attach(beam, result)
        elif beam_specifier.beam_each_division:
            for x in result:
                beam = spannertools.MultipartBeam()
                attach(beam, x)
        result = [selectiontools.Selection(x) for x in result]
        return result

    def _make_numeric_map_part(
        self,
        numerator,
        prefix,
        suffix,
        is_note_filled=True,
        ):
        prefix_weight = mathtools.weight(prefix)
        suffix_weight = mathtools.weight(suffix)
        middle = numerator - prefix_weight - suffix_weight
        if numerator < prefix_weight:
            weights = [numerator]
            prefix = sequencetools.split_sequence(
                prefix, weights, cyclic=False, overhang=False)[0]
        middle = self._make_middle_of_numeric_map_part(middle)
        suffix_space = numerator - prefix_weight
        if suffix_space <= 0:
            suffix = ()
        elif suffix_space < suffix_weight:
            weights = [suffix_space]
            suffix = sequencetools.split_sequence(
                suffix,
                weights,
                cyclic=False,
                overhang=False,
                )[0]
        numeric_map_part = prefix + middle + suffix
        return [durationtools.Duration(x) for x in numeric_map_part]

    def _make_output_incised_numeric_map(
        self,
        duration_pairs,
        prefix_talea,
        prefix_lengths,
        suffix_talea,
        suffix_lengths,
        extra_counts_per_division,
        ):
        numeric_map, prefix_talea_index, suffix_talea_index = [], 0, 0
        prefix_length, suffix_length = prefix_lengths[0], suffix_lengths[0]
        prefix = prefix_talea[
            prefix_talea_index:prefix_talea_index+prefix_length]
        suffix = suffix_talea[
            suffix_talea_index:suffix_talea_index+suffix_length]
        if len(duration_pairs) == 1:
            prolation_addendum = extra_counts_per_division[0]
            if isinstance(duration_pairs[0], mathtools.NonreducedFraction):
                numerator = duration_pairs[0].numerator
            else:
                numerator = duration_pairs[0][0]
            numerator += (prolation_addendum % numerator)
            numeric_map_part = self._make_numeric_map_part(
                numerator, prefix, suffix)
            numeric_map.append(numeric_map_part)
        else:
            prolation_addendum = extra_counts_per_division[0]
            if isinstance(duration_pairs[0], tuple):
                numerator = duration_pairs[0][0]
            else:
                numerator = duration_pairs[0].numerator
            numerator += (prolation_addendum % numerator)
            numeric_map_part = self._make_numeric_map_part(
                numerator, prefix, ())
            numeric_map.append(numeric_map_part)
            for i, duration_pair in enumerate(duration_pairs[1:-1]):
                prolation_addendum = extra_counts_per_division[i+1]
                if isinstance(duration_pair, tuple):
                    numerator = duration_pair[0]
                else:
                    numerator = duration_pair.numerator
                numerator += (prolation_addendum % numerator)
                numeric_map_part = self._make_numeric_map_part(
                    numerator, (), ())
                numeric_map.append(numeric_map_part)
            try:
                prolation_addendum = extra_counts_per_division[i+2]
            except UnboundLocalError:
                prolation_addendum = extra_counts_per_division[1+2]
            if isinstance(duration_pairs[-1], tuple):
                numerator = duration_pairs[-1][0]
            else:
                numerator = duration_pairs[-1].numerator
            numerator += (prolation_addendum % numerator)
            numeric_map_part = self._make_numeric_map_part(
                numerator, (), suffix)
            numeric_map.append(numeric_map_part)
        return numeric_map

    def _numeric_map_and_talea_denominator_to_leaf_selections(
        self, numeric_map, lcd):
        from abjad.tools import rhythmmakertools
        selections = []
        specifier = self.duration_spelling_specifier
        if specifier is None:
            specifier = rhythmmakertools.DurationSpellingSpecifier()
        for numeric_map_part in numeric_map:
            selection = scoretools.make_leaves_from_talea(
                numeric_map_part,
                lcd,
                forbidden_written_duration=\
                    specifier.forbidden_written_duration,
                decrease_durations_monotonically=\
                    specifier.decrease_durations_monotonically,
                )
            selections.append(selection)
        return selections

    def _prepare_input(self, seeds):
        helper_functions = self.helper_functions or {}
        prefix_talea = self.incise_specifier.prefix_talea or ()
        helper = helper_functions.get('prefix_talea')
        helper = self._none_to_trivial_helper(helper)
        prefix_talea = helper(prefix_talea, seeds)
        prefix_talea = datastructuretools.CyclicTuple(prefix_talea)

        prefix_lengths = self.incise_specifier.prefix_lengths or ()
        helper = helper_functions.get('prefix_lengths')
        helper = self._none_to_trivial_helper(helper)
        prefix_lengths = helper(prefix_lengths, seeds)
        prefix_lengths = datastructuretools.CyclicTuple(prefix_lengths)

        suffix_talea = self.incise_specifier.suffix_talea or ()
        helper = helper_functions.get('suffix_talea')
        helper = self._none_to_trivial_helper(helper)
        suffix_talea = helper(suffix_talea, seeds)
        suffix_talea = datastructuretools.CyclicTuple(suffix_talea)

        suffix_lengths = self.incise_specifier.suffix_lengths or ()
        helper = helper_functions.get('suffix_lengths')
        helper = self._none_to_trivial_helper(helper)
        suffix_lengths = helper(suffix_lengths, seeds)
        suffix_lengths = datastructuretools.CyclicTuple(suffix_lengths)

        extra_counts_per_division = self.extra_counts_per_division or ()
        helper = helper_functions.get('extra_counts_per_division')
        helper = self._none_to_trivial_helper(helper)
        extra_counts_per_division = helper(extra_counts_per_division, seeds)
        if extra_counts_per_division:
            extra_counts_per_division = datastructuretools.CyclicTuple(
                extra_counts_per_division)
        else:
            extra_counts_per_division = datastructuretools.CyclicTuple([0])

        split_divisions_by_counts = self.split_divisions_by_counts or ()
        helper = helper_functions.get('split_divisions_by_counts')
        helper = self._none_to_trivial_helper(helper)
        split_divisions_by_counts = helper(split_divisions_by_counts, seeds)
        split_divisions_by_counts = datastructuretools.CyclicTuple(
            split_divisions_by_counts)

        return (
            prefix_talea,
            prefix_lengths,
            suffix_talea,
            suffix_lengths,
            extra_counts_per_division,
            split_divisions_by_counts,
            )

    ### PUBLIC PROPERTIES ###

    @property
    def extra_counts_per_division(self):
        r'''Gets prolation addenda of incised rhythm-maker.

        Returns tuple or none.
        '''
        return self._extra_counts_per_division

    @property
    def helper_functions(self):
        r'''Gets helper functions of incised rhythm-maker.

        Returns dictionary or none.
        '''
        return self._helper_functions

    @property
    def incise_specifier(self):
        r'''Gets incise specifier or incised rhythm-maker.

        ..  container:: example

            Output-incised notes:

            ::

                >>> incise_specifier = rhythmmakertools.InciseSpecifier(
                ...     incise_output=True,
                ...     prefix_talea=(-8, -7),
                ...     prefix_lengths=(2,),
                ...     suffix_talea=(-3,),
                ...     suffix_lengths=(4,),
                ...     talea_denominator=32,
                ...     )
                >>> maker = rhythmmakertools.IncisedRhythmMaker(
                ...     incise_specifier=incise_specifier,
                ...     )

            ::

                >>> divisions = [(5, 8), (5, 8), (5, 8)]
                >>> music = maker(divisions)
                >>> lilypond_file = rhythmmakertools.make_lilypond_file(
                ...     music,
                ...     divisions,
                ...     )
                >>> show(lilypond_file) # doctest: +SKIP

            ..  doctest::

                >>> staff = maker._get_rhythmic_staff(lilypond_file)
                >>> f(staff)
                \new RhythmicStaff {
                    {
                        \time 5/8
                        r4
                        r8..
                        c'8 ~ [
                        c'32 ]
                    }
                    {
                        c'2 ~
                        c'8
                    }
                    {
                        c'4
                        r16.
                        r16.
                        r16.
                        r16.
                    }
                }

        ..  container:: example

            Output-incised rests:

            ::

                >>> incise_specifier = rhythmmakertools.InciseSpecifier(
                ...     incise_output=True,
                ...     prefix_talea=(7, 8),
                ...     prefix_lengths=(2,),
                ...     suffix_talea=(3,),
                ...     suffix_lengths=(4,),
                ...     talea_denominator=32,
                ...     fill_with_notes=False,
                ...     )
                >>> maker = rhythmmakertools.IncisedRhythmMaker(
                ...     incise_specifier=incise_specifier,
                ...     )

            ::

                >>> divisions = [(5, 8), (5, 8), (5, 8)]
                >>> music = maker(divisions)
                >>> lilypond_file = rhythmmakertools.make_lilypond_file(
                ...     music,
                ...     divisions,
                ...     )
                >>> show(lilypond_file) # doctest: +SKIP

            ..  doctest::

                >>> staff = maker._get_rhythmic_staff(lilypond_file)
                >>> f(staff)
                \new RhythmicStaff {
                    {
                        \time 5/8
                        c'8..
                        c'4
                        r8
                        r32
                    }
                    {
                        r2
                        r8
                    }
                    {
                        r4
                        c'16. [
                        c'16.
                        c'16.
                        c'16. ]
                    }
                }

        Returns incise specifier or none.
        '''
        return self._incise_specifier

    @property
    def split_divisions_by_counts(self):
        r'''Gets secondary divisions of incised rhythm-maker.

        Returns tuple or none.
        '''
        return self._split_divisions_by_counts

    ### PUBLIC METHODS ###

    def reverse(self):
        r'''Reverses incised rhythm-maker.

        Returns newly constructed rhythm-maker.
        '''
        from abjad.tools import rhythmmakertools
        extra_counts_per_division = \
            self._reverse_tuple(self.extra_counts_per_division)
        split_divisions_by_counts = \
            self._reverse_tuple(self.split_divisions_by_counts)
        duration_spelling_specifier = self.duration_spelling_specifier
        if duration_spelling_specifier is None:
            duration_spelling_specifier = \
                rhythmmakertools.DurationSpellingSpecifier()
        duration_spelling_specifier = duration_spelling_specifier.reverse()
        incise_specifier = self.incise_specifier
        if incise_specifier is not None:
            incise_specifier = incise_specifier.reverse()
        maker = new(
            self,
            duration_spelling_specifier=duration_spelling_specifier,
            extra_counts_per_division=extra_counts_per_division,
            incise_specifier=incise_specifier,
            split_divisions_by_counts=split_divisions_by_counts,
            )
        return maker

    def rotate(self, n=0):
        r'''Rotates incised rhythm-maker.

        Returns newly constructed rhythm-maker.
        '''
        extra_counts_per_division = \
            self._rotate_tuple(self.extra_counts_per_division, n)
        split_divisions_by_counts = \
            self._rotate_tuple(self.split_divisions_by_counts, n)
        incise_specifier = self.incise_specifier
        if incise_specifier is not None:
            incise_specifier = incise_specifier.rotate(n)
        maker = new(
            self,
            extra_counts_per_division=extra_counts_per_division,
            incise_specifier=incise_specifier,
            split_divisions_by_counts=split_divisions_by_counts,
            )
        return maker



# -*- encoding: utf-8 -*-
from .context_properties import context_properties
from .contexts import contexts
from .current_module import current_module
from .engravers import engravers
from .grob_interfaces import grob_interfaces
from .interface_properties import interface_properties
from .language_pitch_names import language_pitch_names
from .markup_functions import markup_functions, markup_list_functions
from .music_glyphs import music_glyphs



# -*- encoding: utf-8 -*-
import copy


def permute_sequence(sequence, permutation):
    '''Permutes `sequence`.

    ::

        >>> sequencetools.permute_sequence([10, 11, 12, 13, 14, 15], [5, 4, 0, 1, 2, 3])
        [15, 14, 10, 11, 12, 13]

    Returns new object of `sequence` type.
    '''
    from abjad.tools import sequencetools

    if not sequencetools.Sequence(*permutation).is_permutation() or \
        len(sequence) != len(permutation):
        message = '{!r} must be permutation of length {}.'
        message = message.format(permutation, len(sequence))
        raise TypeError(message)

    result = []
    for index in permutation:
        new_element = copy.copy(sequence[index])
        result.append(new_element)
    if isinstance(sequence, str):
        return ''.join(result)
    else:
        return type(sequence)(result)


# -*- encoding: utf-8 -*-
import inspect
import pytest
from abjad.tools import documentationtools


classes = documentationtools.list_all_abjad_classes()
@pytest.mark.parametrize('class_', classes)
def test_abjad___hash___01(class_):
    r'''All concrete classes with __hash__ can hash.
    '''

    if not inspect.isabstract(class_):
        if getattr(class_, '__hash__', None):
            instance = class_()
            value = hash(instance)
            assert isinstance(value, int)


# -*- encoding: utf-8 -*-
from abjad import *


def test_sequencetools_partition_sequence_by_weights_01():

    sequence = [3, 3, 3, 3, 4, 4, 4, 4, 5, 5]
    groups = sequencetools.partition_sequence_by_weights(
        sequence,
        [3, 9],
        cyclic=False,
        overhang=True,
        )
    assert groups == [[3], [3, 3, 3], [4, 4, 4, 4, 5, 5]]


def test_sequencetools_partition_sequence_by_weights_02():

    sequence = [3, 3, 3, 3, 4, 4, 4, 4, 5, 5]
    groups = sequencetools.partition_sequence_by_weights(
        sequence,
        [3, 9],
        cyclic=False,
        overhang=False,
        )
    assert groups == [[3], [3, 3, 3]]


def test_sequencetools_partition_sequence_by_weights_03():

    sequence = [3, 3, 3, 3, 4, 4, 4, 4, 5]
    groups = sequencetools.partition_sequence_by_weights(
        sequence,
        [12],
        cyclic=True,
        overhang=True,
        )
    assert groups == [[3, 3, 3, 3], [4, 4, 4], [4, 5]]


def test_sequencetools_partition_sequence_by_weights_04():

    sequence = [3, 3, 3, 3, 4, 4, 4, 4, 5]
    groups = sequencetools.partition_sequence_by_weights(
        sequence,
        [12],
        cyclic=True,
        overhang=False,
        )
    assert groups == [[3, 3, 3, 3], [4, 4, 4]]


def test_sequencetools_partition_sequence_by_weights_05():

    sequence = [3, 3, 3, 3, 4, 4, 4, 4, 5, 5]
    groups = sequencetools.partition_sequence_by_weights(
        sequence,
        [10, 4],
        cyclic=False,
        overhang=True,
        allow_part_weights=Less,
        )
    assert groups == [[3, 3, 3], [3], [4, 4, 4, 4, 5, 5]]


def test_sequencetools_partition_sequence_by_weights_06():

    sequence = [3, 3, 3, 3, 4, 4, 4, 4, 5, 5]
    groups = sequencetools.partition_sequence_by_weights(
        sequence,
        [10, 4],
        cyclic=False,
        overhang=False,
        allow_part_weights=Less,
        )
    assert groups == [[3, 3, 3], [3]]


def test_sequencetools_partition_sequence_by_weights_07():

    sequence = [3, 3, 3, 3, 4, 4, 4, 4, 5, 5]
    groups = sequencetools.partition_sequence_by_weights(
        sequence,
        [10, 5],
        cyclic=True,
        overhang=True,
        allow_part_weights=Less,
        )
    assert groups == [[3, 3, 3], [3], [4, 4], [4], [4, 5], [5]]


def test_sequencetools_partition_sequence_by_weights_08():

    sequence = [3, 3, 3, 3, 4, 4, 4, 4, 5]
    groups = sequencetools.partition_sequence_by_weights(
        sequence,
        [10, 5],
        cyclic=True,
        overhang=False,
        allow_part_weights=Less,
        )
    assert groups == [[3, 3, 3], [3], [4, 4], [4]]


def test_sequencetools_partition_sequence_by_weights_09():

    sequence = [3, 3, 3, 3, 4, 4, 4, 4, 5, 5]
    groups = sequencetools.partition_sequence_by_weights(
        sequence,
        [10, 4],
        cyclic=False,
        overhang=True,
        allow_part_weights=More,
        )
    assert groups == [[3, 3, 3, 3], [4], [4, 4, 4, 5, 5]]


def test_sequencetools_partition_sequence_by_weights_10():

    sequence = [3, 3, 3, 3, 4, 4, 4, 4, 5, 5]
    groups = sequencetools.partition_sequence_by_weights(
        sequence,
        [10, 4],
        cyclic=False,
        overhang=False,
        allow_part_weights=More,
        )
    assert groups == [[3, 3, 3, 3], [4]]


def test_sequencetools_partition_sequence_by_weights_11():

    sequence = [3, 3, 3, 3, 4, 4, 4, 4, 5, 5]
    groups = sequencetools.partition_sequence_by_weights(
        sequence,
        [10, 4],
        cyclic=True,
        overhang=True,
        allow_part_weights=More,
        )
    assert groups == [[3, 3, 3, 3], [4], [4, 4, 4], [5], [5]]


def test_sequencetools_partition_sequence_by_weights_12():

    sequence = [3, 3, 3, 3, 4, 4, 4, 4, 5, 5]
    groups = sequencetools.partition_sequence_by_weights(
        sequence,
        [10, 4],
        cyclic=True,
        overhang=False,
        allow_part_weights=More,
        )
    assert groups == [[3, 3, 3, 3], [4], [4, 4, 4], [5]]


# -*- encoding: utf-8 -*-
import sys
from abjad import *


def test_systemtools_IOManager_count_function_calls_01():

    result = systemtools.IOManager.count_function_calls(
        "Note('c4')",
        globals(),
        )
    if sys.version_info[0] == 2:
        assert result < 11000
    else:
        assert result < 17000


def test_systemtools_IOManager_count_function_calls_02():

    result = systemtools.IOManager.count_function_calls(
        "Note(-12, (1, 4))",
        globals(),
        )
    if sys.version_info[0] == 2:
        assert result == 182
    else:
        assert result == 202


# -*- encoding: utf-8 -*-
from abjad.tools import markuptools
from abjad.tools import pitchtools
from abjad.tools.topleveltools import attach
from abjad.tools.topleveltools import iterate


def label_vertical_moments_in_expr_with_numbered_pitch_classes(
    expr, markup_direction=Down):
    r'''Label pitch-classes of every vertical moment in `expr`:

    ::

        >>> score = Score([])
        >>> staff = Staff("c'8 d'8 e'8 f'8")
        >>> score.append(staff)
        >>> staff = Staff(r"""\clef "alto" g4 f4""")
        >>> score.append(staff)
        >>> staff = Staff(r"""\clef "bass" c,2""")
        >>> score.append(staff)

    ::

        >>> labeltools.label_vertical_moments_in_expr_with_numbered_pitch_classes(
        ...     score)

    ..  doctest::

        >>> print(format(score))
        \new Score <<
            \new Staff {
                c'8
                d'8
                    _ \markup {
                        \small
                            \column
                                {
                                    7
                                    2
                                    0
                                }
                        }
                e'8
                f'8
                    _ \markup {
                        \small
                            \column
                                {
                                    5
                                    0
                                }
                        }
            }
            \new Staff {
                \clef "alto"
                g4
                f4
                    _ \markup {
                        \small
                            \column
                                {
                                    5
                                    4
                                    0
                                }
                        }
            }
            \new Staff {
                \clef "bass"
                c,2
                    _ \markup {
                        \small
                            \column
                                {
                                    7
                                    0
                                }
                        }
            }
        >>

    ::

        >>> show(score) # doctest: +SKIP

    Returns none.
    '''

    for vertical_moment in \
        iterate(expr).by_vertical_moment():
        leaves = vertical_moment.leaves
        pitches = pitchtools.PitchSegment.from_selection(leaves)
        if not pitches:
            continue
        pitch_classes = [
            pitch.numbered_pitch_class.pitch_class_number
            for pitch in pitches
            ]
        pitch_classes = list(set(pitch_classes))
        pitch_classes.sort()
        pitch_classes.reverse()
        pitch_classes = ' '.join([str(x) for x in pitch_classes])
        pitch_classes = r'\small \column { %s }' % pitch_classes
        markup = markuptools.Markup(pitch_classes, markup_direction)
        attach(markup, vertical_moment.start_leaves[-1])
